function v = brl_trig_poly( nd, K, D, ctype )

% Generates the variables of a Bounded Real Lemma with trigonometric
% polynomials: || H(omega) ||_inf <= gamma, for omega belonging to a domain D.
% H(z) is a causal trigonometric polynomial with matrix coefficients.
% Input:
%    nd     - the degree of the polynomial, a vector whose length is the
%             number of dimensions. Each element is the degree on that
%             dimension.
%    K      - the size of the matrix coefficients, a vector with two
%             elements (number of rows and columns). Can be a scalar if the
%             matrix is square.
%    D      - a description of the domain. If missing, the inequality holds for all omega.
%             Univariate polynomials: D is an interval [a,b]
%               represented by a vector with 2 elements.
%             Multivariate: D is a cell vector, each element being a structure.
%               D{i}.p - symmetric polynomial represented in vector form (see pos_trig_poly).
%               D{i}.d - is the degree of the polynomial
%    ctype  - a string describing the coefficients
%             'r' or 'real' - real coefficients (default type)
%             'c' or 'complex' - complex coefficients
% Output:
%    v      - a vector with v(1)=gamma^2, and v(2:end) the coefficients of H.
%             Each matrix coefficient is vectorized (on columns).

% BD 16.01.2009

% check arguments !!!
if nargin < 2
  K = 1;
end
if nargin < 3
  D = [];   % global positivity
end
if nargin < 4
  ctype = 'r';   % real coefficients
end
nvar = length(nd);
dom_pos = ~isempty(D);   % positivity on domain (as opposed to global)
if prod(nd) < 4 && dom_pos
  error('The order of the polynomial must be at least 4')  % ????
end
if nvar == 1 && dom_pos && length(D) ~= 2
  error('The variable D must contain two values, representing an interval')
end
real_coef = 1;
if ctype(1) == 'c'
  real_coef = 0;
end

% always use two variables for the size of matrix coefficients
if length(K) == 1
  K = [K K];
end

% some numbers
K2 = K(1)*K(2);     % number of elements in a matrix coefficient
K1 = K(1)*K(1);     % number of elements in a matrix coef. in BRL polynomial equality
K1s = K(1)*(K(1)+1)/2;  % same, in symmetric part
Rsize = nd + 1;
N = prod(Rsize);    % number of block coefficients
nH = N * K2;        % number of filter coefficients
NK1 = N*K(1);       % size of Gram matrix Q0
nQ0 = NK1 + K(2);   % size of extended Q0 matrix

if dom_pos
  if nvar == 1    % univariate
    if real_coef
      nQ1 = (Rsize - 2)*K(1);
      c0 = - 0.5 - cos(D(1))*cos(D(2));
      c1 = 0.5*( cos(D(1)) + cos(D(2)) );
      c2 = - 0.25;
    else              % complex coefficients
      nQ1 = (Rsize - 1)*K(1);
      c0 = -cos((D(2)-D(1))/2);
      c1 = 0.5*exp(sqrt(-1)*(D(1)+D(2))/2);
    end
  else            % multivariate
    nDpol = length(D);    % number of polynomials describing the domain
    ND = zeros(1,nDpol);
    NS1 = zeros(1,nDpol);
    LS1 = zeros(1,nDpol);
    %nQ1 = zeros(1,nDpol);
    for i = 1 : nDpol     % check
      if length(D{i}.d) ~= nvar
        error('Number of variables must be the same for all polynomials')
      end
      Dsize{i} = D{i}.d + 1;
      ND(i) = (prod( 2*D{i}.d + 1 ) + 1) / 2; % number of distinct coefficients
      if length( D{i}.p ) ~= ND(i)
        error('Number of coefficients of constraint polynomial does no correspond with the degree')
      end
      D{i}.p = D{i}.p(:);
      dS1{i} = nd - D{i}.d;   % degree of sos multiplying this constraint polynomial
      NS1(i) = (prod( 2*dS1{i} + 1 ) + 1) / 2;  % number of matrix coefficients of sos
      %nQ1(i) = prod(dS1{i}+1) * K(1);   % size of corresponding Gram matrix
      LS1(i) = (NS1(i)-1)*K1 + K1s;   % length of vectorized sos
    end
    %vnQ1 = [0 cumsum(nQ1.*nQ1)];   % delimitators in concatenated vectorized Q1 matrices
    %lenQ1 = vnQ1(end);        % total length of vectorized Q1 matrices
    vnS1 = [0 cumsum(LS1)];  % delimitators in concatenated vectorized sos
    lenS1 = vnS1(end);       % total length of vectorized sos
    
    % build convolution matrices for symmetric polynomial multiplications D(z) * S1(z)
    NQ = (prod( 2*nd+1 ) + 1) / 2;  % number of distinct matrix coefficients of symmetric "pol" gamma^2*I
    for i = 1 : nDpol
      NsS1(i) = prod( 2*dS1{i} + 1 );  % number of matrix coefficients of symmetric sos
      NsD(i) = prod( 2*D{i}.d + 1 );   % number of coefficients of symmetric D
      C1{i} = sparse(NQ, NsS1(i));
      Dsym = [conj(D{i}.p(end:-1:2)); D{i}.p];
      for id = 1 : NsD(i)
        for is = 1 : NsS1(i)
          eD = ind2exp_nd(id, 2*D{i}.d);
          eS = ind2exp_nd(is, 2*dS1{i});
          ip = exp2ind_nd(eD + eS + 1, 2*nd) - NQ + 1;
          if ip > 0     % compute only "half" of convolution matrix
            C1{i}(ip, is) = Dsym(id);
          end
        end
      end
    end
  end
end

% store Toeplitz matrices
dmax = max(Rsize);
for i = 1 : 2*dmax-1
  T0{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
end
if dom_pos && nvar == 1   % 1D case: precompute matrices that affect Q1
  if real_coef
    for i = 1 : Rsize+4    % for defining S1
      T1{i} = spdiags( ones(Rsize-2,1), i-3, Rsize-2, Rsize-2 );
    end
    for i = 1 : Rsize
      C1{i} = c2*T1{i} + c1*T1{i+1} + c0*T1{i+2} + c1*T1{i+3} + c2*T1{i+4};
    end
  else      % complex coefficients
    for i = 1 : Rsize+2    % for defining S1
      T1{i} = spdiags( ones(Rsize-1,1), i-2, Rsize-1, Rsize-1 );
    end
    for i = 1 : Rsize
      C1{i} = c1*T1{i} + c0*T1{i+1} + conj(c1)*T1{i+2};
    end
  end
end

% store block selectors
for i = 1 : K(1)
  for j = 1 : K(1)
    blS{i,j} = zeros(K(1));
    blS{i,j}(i,j) = 1;
  end
end

cvx_begin set
  if real_coef
    variable v(1+nH);     % the BRL variable
    variable Q0( nQ0,nQ0 ) symmetric;
    Q0 == semidefinite( nQ0 );   % Gram matrix for S0
  else
    variable v(1+nH) complex;
    variable Q0( nQ0,nQ0 ) hermitian;
    Q0 == hermitian_semidefinite( nQ0 );
  end
  if dom_pos
    if nvar == 1
      if real_coef
        variable Q1( nQ1,nQ1 ) symmetric;
        Q1 == semidefinite( nQ1 );
      else
        variable Q1( nQ1,nQ1 ) hermitian;
        Q1 == hermitian_semidefinite( nQ1 );
      end
    else    % multivariate
%       variable Q1(lenQ1);   % vectorized Q1 matrices
%       for i = 1 : nDpol     % declare each matrix as positive semidefinite
%         reshape(Q1(vnQ1(i)+1 : vnQ1(i+1)), nQ1(i), nQ1(i)) == semidefinite(nQ1(i));
%       end
      % !!!! alternative solution
      %     evalin('caller', ['variable Q1_', num2str(i), '(', num2str(nQ1(i)), ',',num2str(nQ1(i)), ') symmetric;'])
      variable S1(lenS1);   % vectorized sos
      for i = 1 : nDpol     % declare each sos
        S1(vnS1(i)+1 : vnS1(i+1)) == sos_trig_poly(dS1{i}, K(1));
      end
    end
  end

  % equalities for gamma^2 I = S0(z) + D(z)*S1(z)
  ki = zeros(nvar,1);
  kb = 1;
  k = 1;
  Ns = (prod( 2*nd+1 ) + 1) / 2;     % number of block coefs in symmetric polynomial
  for kk = 1 : Ns
    T = T0{ki(1)+dmax}(1:Rsize(1),1:Rsize(1));
    for i = 2 : nvar
      T = kron( T0{ki(i)+dmax}(1:Rsize(i),1:Rsize(i)), T );
    end
    %-------------------------------------
    % multivariate: build vectors needed in the linear combination with S1
    if dom_pos && nvar > 1
      for id = 1 : nDpol      % for each polynomial describing the domain
        ca{id} = C1{id}(kb,NS1(id)-1:-1:1); % the anticausal part of current row
        c0{id} = C1{id}(kb,NS1(id));        % central element
        cc{id} = C1{id}(kb,NS1(id)+1:end);  % causal part
      end
    end
    %-------------------------------------
    for j = 1 : K(1)   % for the elements of a block, by columns
      for i = 1 : K(1)
        TT = kron(T,blS{i,j});
        %---------------------------------
        % multivariate: build big vectors multiplying the sos S1
        if dom_pos && nvar > 1
          vcC1 = zeros(1, lenS1);   % will multiply S1
          vaC1 = zeros(1, lenS1);   % will multiply conj(S1)
          for id = 1 : nDpol
            % for multiplication with non-central blocks
            vcC1(vnS1(id) + K1s + 1 : vnS1(id+1)) = kron(cc{id}, vec(blS{i,j}));
            vaC1(vnS1(id) + K1s + 1 : vnS1(id+1)) = kron(ca{id}, vec(blS{j,i}));
            % for the central block
            if i >= j
              vcC1(vnS1(id) + K(1)*(j-1) - j*(j-1)/2 + i) = c0{id};
            else
              vaC1(vnS1(id) + K(1)*(i-1) - i*(i-1)/2 + j) = c0{id};
            end
          end
        end
        %---------------------------------
        if kb==1 && i==j    % where gamma appears
          if dom_pos   % positivity on interval
            if nvar == 1
              v(1) == trace( TT * Q0(1:NK1,1:NK1)) ...
                    + trace( kron(C1{k}, blS{i,j}) * Q1);
            else  % multivariate
              v(1) == trace( TT * Q0(1:NK1,1:NK1)) + vcC1 * S1 + vaC1 * conj(S1);
            end
          else     % global positivity
            v(1) == trace( TT * Q0(1:NK1,1:NK1));
          end
          k = k+1;
        elseif kb>1 || i>j  % other elements (symmetry accounted for)
          if dom_pos   % positivity on interval
            if nvar == 1
              trace( TT * Q0(1:NK1,1:NK1)) ...
               + trace( kron(C1{k}, blS{i,j}) * Q1) == 0;
            else  % multivariate
              trace( TT * Q0(1:NK1,1:NK1)) + vcC1 * S1 + vaC1 * conj(S1) == 0;
            end
          else     % global positivity
            trace( TT * Q0(1:NK1,1:NK1)) == 0;
          end
          k = k+1;
        end
      end
    end

    kb = kb + 1;
    ki(1) = ki(1) + 1;        % go to next (poly)index
    for i = 1 : nvar - 1
      if ki(i) == Rsize(i)
        ki(i) = -Rsize(i)+1;
        ki(i+1) = ki(i+1) + 1;
      else
        break;
      end
    end
  end

  % equalities for [Q0 h; h' I] >= 0
  % coefficients of H
  k = 1;
  for i = 1 : N
    for j = 1 : K(2)
      v(k+1:k+K(1)) == Q0( (i-1)*K(1)+1 : i*K(1), NK1+j );
      k = k + K(1);
    end
  end
  % unit matrix
  Q0(NK1+1:end, NK1+1:end) == eye(K(2));
 
cvx_end
