function [h,g] = cocof(n, ws, gs, nreg)
% Design complex symmetric compaction filter.
% Minimize stopband energy with stopband error bound.
% Input:
%   n      - filter order
%   ws     - stopband edge (normalized frequency)
%   gs     - max stopband amplitude
%   nreg   - number of regularity constraints
% Output:
%   h      - filter coefficients
%   g      - product filter

% BD 16.06.2006

if rem(n,2) == 0
  error('Filter order must be odd')
end
if nargin < 4
  nreg = 0;
end
  
n1 = n+1;       % length
nh1 = n1/2;
nh = nh1 - 1;   % half order
NH1 = nh1 * nh1;

% stopband energy coefficients
cc = zeros(n1,1);
cc(1) = 1 - ws;
nn = (1:n)';
cc(2:end) = - 2 * sin(pi*ws*nn) ./ (pi*nn);

% SDP problem description
Ks.f = n1;              % free variables: product filter coefs
Ks.s = [nh1 nh1 nh1];   % Gram matrices:
                        % 1 for global positivity, 2 for positivity on [ws,pi]

nrA = n1 + nh1 + n1 + nreg; % constraints:
                            %   coefs + Nyquist + pos on [ws,pi] + regularity
ncA = n1 + 3*NH1;   % number of variables

As = zeros(nrA,ncA);
bs = zeros(nrA,1);  % right hand term of the equality constraints
cs = zeros(ncA,1);
cs(1:n1) = cc;      % objective: minimize stopband energy

% Gram matrix description
As(1:n1, 1:n1) = - eye(n1);   % coefs
nc = n1+1 : n1+NH1;              % column range for matrix coefficients
% Toeplitz part
for k = 1 : nh1
  As(k,nc) = 0.5 * vec( diag(ones(nh1-k+1,1),k-1) )';
end
% Hankel part
warning off MATLAB:hankel:AntiDiagonalConflict
for k = 2 : n1
  v = zeros(n,1);
  v(k-1) = 1;
  As(k,nc) = As(k,nc) + 0.25 * vec( hankel(v(1:nh1), v(nh1:end)) )';
end

% Nyquist constraints
bs(n1+1) = 1;
for k = 1 : nh1
  As(n1+k, 2*k-1) = 1;
end

% positivity on [ws,pi]
ir = n1 + nh1;    % start row (-1) in As
As(ir+1:ir+n1, 1:n1) = eye(n1);
nc1 = n1+NH1+1 : n1+2*NH1;  % column range for Q1 and Q2
nc2 = n1+2*NH1+1 : n1+3*NH1;
bs(ir+1) = 2*gs*gs;         % max stopband error

a = cos(pi*ws);
b = -1;   % cos(pi)
for i1 = 1 : nh1
  for i2 = 1 : nh1
    T = zeros(nh1);
    T(i1,i2) = 0.5;
    v = vec(T)';
    k = i1 + i2 - 1;
    As(ir+k, nc1) = As(ir+k, nc1) - b * v;
    As(ir+k, nc2) = As(ir+k, nc2) + a * v;
    k = abs(i1 - i2) + 1;
    As(ir+k, nc1) = As(ir+k, nc1) - b * v;
    As(ir+k, nc2) = As(ir+k, nc2) + a * v;
    k = i1 + i2;
    As(ir+k, nc1) = As(ir+k, nc1) + 0.5 * v;
    As(ir+k, nc2) = As(ir+k, nc2) - 0.5 * v;
    k = abs(i1+i2-3) + 1;
    As(ir+k, nc1) = As(ir+k, nc1) + 0.5 * v;
    As(ir+k, nc2) = As(ir+k, nc2) - 0.5 * v;
    k = abs(i1-i2+1) + 1;
    As(ir+k, nc1) = As(ir+k, nc1) + 0.5 * v;
    As(ir+k, nc2) = As(ir+k, nc2) - 0.5 * v;
    k = abs(i1-i2-1) + 1;
    As(ir+k, nc1) = As(ir+k, nc1) + 0.5 * v;
    As(ir+k, nc2) = As(ir+k, nc2) - 0.5 * v;
  end
end
As(ir+2:ir+n1, nc1) = 0.5 * As(ir+2:ir+n1, nc1);
As(ir+2:ir+n1, nc2) = 0.5 * As(ir+2:ir+n1, nc2);

% call SeDuMi
%pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-8;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

g = x(1:n1);      % product filter

h = g;            % temporary

% spectral factorization yielding linear phase factor
h = factsp_orch(g);  % usual spectral factorization, then root sorting
z = roots(h);
p = find(imag(z) < 0);
z(p) = 1./conj(z(p));
h = poly(z)';
h = h/norm(h);
