function [X, hinf_err, E] = fir2_deconv_hinf_cvx(G, D, Nx, nplus)

% Find optimal Hinf solution to 2D FIR deconvolution problem.
% Implementation with CVX.
% Input:
%   G         - given channel model (coefficient matrix)
%   D         - desired model (coefficient matrix)
%   Nx        - order of FIR equalizer
%   nplus     - relaxation degree is Nx+nplus
% Output:
%   H         - optimal filter
%   hinf_err  - optimal error norm
%   E         - error 2D FIR system

% BD 9.05.2010

if nargin < 4
  nplus = [0 0];
end

nbig = Nx + nplus;          % degree of polynomial padded with zeros
N = prod(nbig+1);           % number of distinct coefficients
zmask = zeros(nbig+1);      % a mask for forcing extra coefficients to zero
zmask(1:Nx(1)+1, 1:Nx(2)+1) = ones(Nx+1);
zmask = vec(zmask);
zpos = find(zmask == 0);    % positions of zero elements

% build matrix representing fixed "A" in the fitting problem
sG = size(G);
dG = sG - 1;
nG = prod(sG);
A = zeros(2*nG,1);
A(1:2:end) = vec(G);
A(2) = 1;

% "B"
dP = nbig + dG;   % degree of the product X*A
sP = dP+1;
nP = prod(sP);
Dbig = zeros(sP);
Dbig(1:sG(1),1:sG(2)) = D;
B = zeros(2*nP,1);
B(1:2:end) = vec(Dbig);

cvx_begin
  variable Xbig(N);
  variable v(2*nP+1);
  minimize v(1)
  subject to
    v == brl_trig_poly(dP,[1 2]);
    v(2:end) + B == pol_mult_vec_c(Xbig, nbig, A, dG, 1, [1 2], 1);
    Xbig(zpos) == 0;    % force extra coefficients to zero
cvx_end
Xbig = reshape(Xbig, nbig(1)+1, nbig(2)+1);
X = Xbig(1:Nx(1)+1, 1:Nx(2)+1);
hinf_err = sqrt(v(1));

i = 2;
for i1 = 1 : sP(1)
  for i2 = 1 : sP(2)
    E{i1,i2} = [v(i) v(i+1)];
    i = i + 2;
  end
end
