function hinfn = fir2_hinf_norm(H, nplus)

% Compute (approximate) Hinf norm of a 2D FIR filter
% Method: BRL lemma based on sum-of-squares relaxation
% Input:
%   H     - matrix of filter coefficients
%   nplus - extra degree for relaxation (implicit value is zero)
% Output:
%   hinfn - Hinf norm (an upper bound, actually) of the filter

% BD 22.08.2005

if nargin < 2
  nplus = [0 0];
end

%---------------------------------------------------------
% extend size if necessary
[nt1, nt2] = size(H);
nh1 = nt1 + nplus(1);
nh2 = nt2 + nplus(2);
Ht = zeros(nh1, nh2);
Ht(1:nt1, 1:nt2) = H;

% here starts the SDP problem description
N = nh1 * nh2;    % number of filter coefficients
Nq = N+1;
Nqs = Nq*Nq;
M = N + (nh1-1)*(nh2-1);    % number of "trace" constraints

nrA = Nq + M;     % number of equality constraints
ncA = 1 + Nqs;    % number of variables

Ks.l = 1;         % stopband attenuation (squared)
Ks.s = [Nq];      % the BRL matrix

As = spalloc( nrA, ncA, ceil(nrA*ncA/20) );  % allocate some space...
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );
cs(1) = 1;              % minimization (of stopband attenuation)

% the last row of Q is vec(H)'
hv = vec(Ht);
i = 1;
for k2 = 1 : nh2
  for k1 = 1 : nh1
    Z = spalloc(Nq,Nq,1);
    Z(Nq,i) = 1;
    As(i, 2:end) = vec(Z)';
    bs(i) = hv(i);
    i = i+1;
  end
end

% the lower right corner of Q is 1
Z = spalloc(Nq,Nq,1);
Z(Nq,Nq) = 1;
As(Nq, 2:end) = vec(Z)';
bs(Nq) = 1;

%-------------------------------------
% try speeding-up by storing Toeplitz matrices
dmax = max(nh1,nh2);
for i = 1 : 2*dmax-1
  Ts{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
end
%-------------------------------------

% the "trace" (Q) part of the BRL equalities
k1=0 ; k2=0;
i = Nq+1;
while k1 < nh1 & k2 < nh2
  Z = spalloc(Nq,Nq,Nq);
  Z(1:N,1:N) = kron(Ts{k2+nh2},Ts{k1+nh1});
  As(i, 2:end) = vec(Z)';
  k1 = k1 + 1;
  if k1 == nh1
    k1 = -nh1+1;
    k2 = k2+1;
  end
  i = i+1;
end

% the criterion appears in the main diagonal equality
As(Nq+1,1) = -1;

% call Sedumi
%pars.fid = 0;
pars.eps = 1e-8;

[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info

hinfn = sqrt(x(1));
