function [h, gs] = fir2_inf_alin_cvx(nh, tau, gp, Dp, nDp, Ds, nDs, nplus)

% Design linear phase 2D FIR filter with Hinf constraints.
% The passband and stopband are defined by the positivity
% of some trigonometric polynomials.
% The stopband error is minimized.
% Input:
%   nh          - degree of the polynomial
%   tau         - desired group delay in passband
%   gp          - passband error bound
%   Dp, Ds      - sets of polynomials defining the passband and the stopband;
%                 each variable is an array of cells; one cell represents
%                 a symmetric polynomial; the coefficients of a polynomial are stored
%                 completely, ignoring symmetry
%   nDp, nDs    - number of polynomials for each domain in passband and stopband;
%                 if e.g. nDp = [3 4], then length(Dp) = 7 and the passband 
%                 is the union of two domains, one defined by 3 polynomials (the
%                 first 3 from Dp) and the other defined by 4 polynomials.
%   nplus       - degree of the relaxation (nh+nplus); implicit value = 0
% Return:
%   h           - coefficients of the optimal filter (symmetric polynomial)
%   gs          - optimal stopband error

% BD 9.05.2010

% checking...
if nargin < 8
  nplus = [0 0];
end
if sum(nDp) ~= length(Dp)
  error('Wrong number of passband polynomials')
end
if sum(nDs) ~= length(Ds)
  error('Wrong number of stopband polynomials')
end

nbig = nh + nplus;          % degree of polynomial padded with zeros
N = prod(nbig+1);           % number of distinct coefficients
zmask = zeros(nbig+1);      % a mask for forcing extra coefficients to zero
zmask(1:nh(1)+1, 1:nh(2)+1) = ones(nh+1);
zmask = vec(zmask);
zpos = find(zmask == 0);    % positions of zero elements

tau_pol = zeros(nbig+1);    % vector for z^{-tau}
tau_pol(tau(1)+1,tau(2)+1) = 1;
tau_pol = vec(tau_pol);

cvx_begin
  variable hbig(N);  % the polynomial (zero padded)
  variable gs;       % stopband attenuation
  minimize gs;
  subject to
    fci = 0;   % index of first polynomial for a constraint
    for i = 1 : length(nDp) % passband constraints
      clear Dcrt;
      for k = 1 : nDp(i)    % convert each polynomial in that constraint
        psize = size(Dp{fci+k});
        Dcrt{k}.d = (psize-1)/2;
        ND = prod(psize);
        v = vec(Dp{fci+k});
        Dcrt{k}.p = v((ND+1)/2:end);
      end
      eval(['variable vp', num2str(i), '(N+1);']);
      eval(['vp', num2str(i), ' ==  brl_trig_poly(nbig,1,Dcrt);']);
      eval(['vp', num2str(i), '(1) == gp*gp']);
      eval(['vp', num2str(i), '(2:end) == hbig - tau_pol']);
      fci = fci + nDp(i);
    end
    
    fci = 0;
    for i = 1 : length(nDs) % stopband constraints
      clear Dcrt;
      for k = 1 : nDs(i)    % convert each polynomial in that constraint
        psize = size(Ds{fci+k});
        Dcrt{k}.d = (psize-1)/2;
        ND = prod(psize);
        v = vec(Ds{fci+k});
        Dcrt{k}.p = v((ND+1)/2:end);
      end
      eval(['variable vs', num2str(i), '(N+1);']);
      eval(['vs', num2str(i), ' ==  brl_trig_poly(nbig,1,Dcrt);']);
      eval(['vs', num2str(i), '(1) == gs']);
      eval(['vs', num2str(i), '(2:end) == hbig']);
      fci = fci + nDs(i);
    end
    
    hbig(zpos) == 0;    % force extra coefficients to zero
cvx_end
hbig = reshape(hbig, nbig(1)+1, nbig(2)+1);
h = hbig(1:nh(1)+1, 1:nh(2)+1);
gs = sqrt(gs);
