function [h, gs] = fir2_inf_lin(nh, gp, Dp, nDp, Ds, nDs, nplus)

% Design linear phase 2D FIR filter with Hinf constraints.
% The passband and stopband are defined by the positivity
% of some trigonometric polynomials.
% The stopband error is minimized.
% Input:
%   nh          - degree of the polynomial
%   gp          - passband error bound
%   Dp, Ds      - sets of polynomials defining the passband and the stopband;
%                 each variable is an array of cells; one cell represents
%                 a symmetric polynomial; the coefficients of a polynomial are stored
%                 completely, ignoring symmetry
%   nDp, nDs    - number of polynomials for each domain in passband and stopband;
%                 if e.g. nDp = [3 4], then length(Dp) = 7 and the passband 
%                 is the union of two domains, one defined by 3 polynomials (the
%                 first 3 from Dp) and the other defined by 4 polynomials
%   nplus       - degree of the relaxation (nh+nplus); implicit value = 0
% Return:
%   h           - coefficients of the optimal filter (symmetric polynomial)
%   gs          - optimal stopband error

% BD 31.01.2006

% checking...
if nargin < 7
  nplus = [0 0];
end
if sum(nDp) ~= length(Dp)
  error('Wrong number of passband polynomials')
end
if sum(nDs) ~= length(Ds)
  error('Wrong number of stopband polynomials')
end

% sizes, orders and numbers...
nt1 = nh(1)+1;
nt2 = nh(2)+1;          % true size of filter (without zero coefficients)
nh1 = nt1 + nplus(1);
nh2 = nt2 + nplus(2);   % complete size (with zero coefs)

nlmi1 = length(nDp);    % number of LMIs for the passband
nlmi2 = length(nDs);    % number of LMIs for the stopband
nlmi = nlmi1 + nlmi2;
nPmat1 = sum(nDp);      % number of P matrices (for passband LMIs)
nPmat2 = sum(nDs);      % number of P matrices (for stopband LMIs)
nD = [nDp nDs];
nPmat = nPmat1 + nPmat2;

nd1 = zeros(2,nPmat1);  % orders of Dp polynomials
for i = 1 : nPmat1
  nd1(1,i) = (size(Dp{i},1)-1) / 2;
  nd1(2,i) = (size(Dp{i},2)-1) / 2;
  D{i} = Dp{i};
end
nd2 = zeros(2,nPmat2);  % orders of Ds polynomials
for i = 1 : nPmat2
  nd2(1,i) = (size(Ds{i},1)-1) / 2;
  nd2(2,i) = (size(Ds{i},2)-1) / 2;
  D{nPmat1+i} = Ds{i};
end
nd = [nd1 nd2];

np1 = repmat([nh1; nh2],1,nPmat1)  - nd1;    % sizes (!) of P polynomials
np2 = repmat([nh1; nh2],1,nPmat2)  - nd2;
np = [np1 np2];

Np1 = np1(1,:) .* np1(2,:);
Np2 = np2(1,:) .* np2(2,:);
Np = [Np1 Np2];
Np1s = Np1.*Np1;
Np2s = Np2.*Np2;
Nps = [Np1s Np2s];


N = nh1 * nh2;    % number of filter coefficients
M = N + (nh1-1)*(nh2-1);    % number of "trace" constraints
                            % (and of distinct filter coefficients)
Mz = M - nt1*nt2 - (nt1-1)*(nt2-1); % number of zero coefficients

Nhs = N*N;

% here starts the SDP problem description
Nlmi = 1 + nlmi1 + 2*nlmi2;
nrA = M*Nlmi + Mz;   % number of equality constraints
                                      % 1 for passband global positivity,
                                      % 1 for each LMI in passband,
                                      % 2 for each LMI in stopband,
                                      % Mz for zero coefs
ncA = 1+M + Nlmi*Nhs + sum(Np1s) + 2*sum(Np2s);     % number of variables

Ks.f = 1 + M;               % stopband attenuation and M filter coefficients
Ks.s = [N];
icAlmi = zeros(1,Nlmi);
icAlmi(1) = 1;
k=0;
for i = 1 : nlmi           % for each LMI
  v = [];
  v = [v N];                % the size of Q matrix
  v = [v Np(k+1:k+nD(i))];  % and of the P matrices
  if i <= nlmi1       % 1 set of matrices for passband LMIs
    icAlmi(i+1) = length(Ks.s)+1;
    Ks.s = [Ks.s v];
  else                % 2 sets of matrices for stopband LMIs
    ii = nlmi1 + 2*(i-nlmi1);
    icAlmi(ii) = length(Ks.s)+1;
    Ks.s = [Ks.s v v];        % twice, due to the absolute value
    icAlmi(ii+1) = icAlmi(ii) + length(v);
  end
  k = k+nD(i);
end

As = spalloc( nrA, ncA, ceil(nrA*ncA/20) );  % allocate some space...
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );
cs(1) = 1;              % minimization (of stopband attenuation)

irAs = 0 : M : Nlmi*M;
icAs = cumsum([0 Ks.s.*Ks.s]) + 1+M;

%-------------------------------------
% try speeding-up by storing Toeplitz matrices
dmax = max(nh1,nh2);
for i = 1 : 2*dmax-1
  Ts{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
end
%-------------------------------------

% the "trace" (Q) part of the positivity equalities
k1=0 ; k2=0;
i = 1;
while k1 < nh1 & k2 < nh2
  Z = spalloc(N,N,N);
  Z = kron(Ts{k2+nh2},Ts{k1+nh1});
  v = vec(Z)';
  As(i, icAs(icAlmi(1))+1:icAs(icAlmi(2))) = v; % global pos Q
  As(i,i+1) = 1;
  for i1 = 1 : nlmi1    % passband Q (for lower bound)
    As(i1*M+i, icAs(icAlmi(i1+1))+1:icAs(icAlmi(i1+1)+1)) = v;
    As(i1*M+i, i+1) = -1;
  end
  for i1 = 1 : nlmi2
    ib = nlmi1 + 2*i1 - 1;
    As(ib*M+i, icAs(icAlmi(ib+1))+1:icAs(icAlmi(ib+1)+1)) = v;
    As(ib*M+i, i+1) = -1;
    As((ib+1)*M+i, icAs(icAlmi(ib+2))+1:icAs(icAlmi(ib+2)+1)) = v;
    As((ib+1)*M+i, i+1) = 1;
  end
  k1 = k1 + 1;
  if k1 == nh1
    k1 = -nh1+1;
    k2 = k2+1;
  end
  i = i+1;
end

% the errors
bs(1) = gp+1;
for i1 = 1 : nlmi1              % passband error is fixed
  bs(i1*M+1) = gp-1;
end
for i1 = 1 : nlmi2
  As((nlmi1 + 2*i1-1)*M+1, 1) = -1;      % stopband attenuation is optimized
  As((nlmi1 + 2*i1)*M+1, 1) = -1;
end

% the "P" part of the BRL equalities
id = 0;
warning off MATLAB:toeplitz:DiagonalConflict
for il = 1 : nlmi       % for each LMI
  for ip = 1 : nD(il)   % for each P matrix in the LMI
    id = id + 1;
    % build the convolution matrices for D polynomials
    clear Td
    tr = zeros(2*np(1,id)-1,1);
    for k2 = 1 : size(D{id},2)
      tc = zeros(2*nh1-1,1);
      tc(1:size(D{id},1)) = D{id}(:,k2);
      Z = toeplitz(tc,tr);
      Td{k2} = sparse(Z);
    end
    
    % the trace matrices for the "P" positive polynomials
    for k2 = 1 : nh2
      Tp{k2} = spalloc(2*np(1,id)-1, Nps(id), Nps(id));
      for p = 1 : 2*np(1,id)-1
        Z = kron(Ts{dmax-nd(2,id)+k2-1}(1:np(2,id), 1:np(2,id)), ...
                 Ts{dmax-np(1,id)+p}(1:np(1,id), 1:np(1,id)));
        Tp{k2}(p,:) = vec(Z)';
      end
    end

    % store in As the "P" part of the BRL equalities
    % the first "half" row (of the half plane)
    z = sparse(2*nh1-1,Nps(id));
    nd22 = size(D{id},2);
    for ii = 1 : nd22
      z = z + Td{ii} * Tp{nd22+1-ii};
    end
    if il <= nlmi1    % passband LMI
      As(il*M+1:il*M+nh1, icAs(icAlmi(il+1)+ip)+1:icAs(icAlmi(il+1)+ip+1) ) = z(nh1:end,:);
    else              % stopband LMI
      ib = nlmi1 + 2*(il-nlmi1) - 1;
      As(ib*M+1:ib*M+nh1, icAs(icAlmi(ib+1)+ip)+1:icAs(icAlmi(ib+1)+ip+1)) = z(nh1:end,:);
      As((ib+1)*M+1:(ib+1)*M+nh1, icAs(icAlmi(ib+2)+ip)+1:icAs(icAlmi(ib+2)+ip+1)) = z(nh1:end,:);
    end

    % the other coefficients
    is = nh1;
    for k2 = 1 : nh2-1
      z = sparse(2*nh1-1,Nps(id));
      for ii = max(1, nd22+1+k2-nh2) : nd22
        z = z + Td{ii} * Tp{nd22+1+k2-ii};
      end
      if il <= nlmi1    % passband LMI
        As(il*M+is+1:il*M+is + 2*nh2-1, icAs(icAlmi(il+1)+ip)+1:icAs(icAlmi(il+1)+ip+1) ) = z;
      else              % stopband LMI
        ib = nlmi1 + 2*(il-nlmi1) - 1;
        As(ib*M+is+1:ib*M+is + 2*nh2-1, icAs(icAlmi(ib+1)+ip)+1:icAs(icAlmi(ib+1)+ip+1)) = z;
        As((ib+1)*M+is+1:(ib+1)*M+is + 2*nh2-1, icAs(icAlmi(ib+2)+ip)+1:icAs(icAlmi(ib+2)+ip+1)) = z;
      end
      is = is + 2*nh2-1;
    end

  end
end

% the forced zero coefficients
k1=0 ; k2=0;
i = 1;
ir = Nlmi*M + 1;
while k1 < nh1 & k2 < nh2
  if abs(k1) >= nt1 | k2 >= nt2   % if zero coefficient
    As(ir, i+1) = 1;
    ir = ir + 1;
  end
  k1 = k1 + 1;
  if k1 == nh1
    k1 = -nh1+1;
    k2 = k2+1;
  end
  i = i+1;
end

% call Sedumi
%pars.fid = 0;
pars.eps = 1e-8;

[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info

gs = x(1);

hbig = zeros(nh1, nh2);
k1=0 ; k2=0;
i = 1;
while k1 < nh1 & k2 < nh2
  hbig( nh1+k1, nh2+k2 ) = x(i+1);
  hbig( nh1-k1, nh2-k2 ) = x(i+1);
  k1 = k1 + 1;
  if k1 == nh1
    k1 = -nh1+1;
    k2 = k2+1;
  end
  i = i+1;
end
h = hbig(nplus(1)+1:2*nh1-nplus(1)-1, nplus(2)+1:2*nh2-nplus(2)-1);