function [h, gs] = fir2_inf_lin_cvx(nh, gp, Dp, nDp, Ds, nDs, nplus)

% Design linear phase 2D FIR filter with Hinf constraints.
% The passband and stopband are defined by the positivity
% of some trigonometric polynomials.
% The stopband error is minimized.
% Input:
%   nh          - degree of the polynomial
%   gp          - passband error bound
%   Dp, Ds      - sets of polynomials defining the passband and the stopband;
%                 each variable is an array of cells; one cell represents
%                 a symmetric polynomial; the coefficients of a polynomial are stored
%                 completely, ignoring symmetry
%   nDp, nDs    - number of polynomials for each domain in passband and stopband;
%                 if e.g. nDp = [3 4], then length(Dp) = 7 and the passband 
%                 is the union of two domains, one defined by 3 polynomials (the
%                 first 3 from Dp) and the other defined by 4 polynomials.
%   nplus       - degree of the relaxation (nh+nplus); implicit value = 0
% Return:
%   h           - coefficients of the optimal filter (symmetric polynomial)
%   gs          - optimal stopband error

% BD 9.05.2010

% checking...
if nargin < 7
  nplus = [0 0];
end
if sum(nDp) ~= length(Dp)
  error('Wrong number of passband polynomials')
end
if sum(nDs) ~= length(Ds)
  error('Wrong number of stopband polynomials')
end

nbig = nh + nplus;          % degree of polynomial padded with zeros
N = (prod(2*nbig+1)+1)/2;   % number of distinct coefficients
zmask = zeros(2*nbig+1);    % a mask for forcing extra coefficients to zero
zmask(nplus(1)+1:end-nplus(1), nplus(2)+1:end-nplus(2)) = ones(2*nh+1);
zmask = vec(zmask);
zmask = zmask(N:-1:1);      % retain only coefficients in halfplane
zpos = find(zmask == 0);    % positions of zero elements

Ipol = eye(N,1);
cvx_begin
  variable hbig(N);  % the polynomial (zero padded)
  variable gs;       % stopband attenuation
  minimize gs;
  subject to
    fci = 0;   % index of first polynomial for a constraint
    for i = 1 : length(nDp) % passband constraints
      clear Dcrt;
      for k = 1 : nDp(i)    % convert each polynomial in that constraint
        psize = size(Dp{fci+k});
        Dcrt{k}.d = (psize-1)/2;
        ND = prod(psize);
        v = vec(Dp{fci+k});
        Dcrt{k}.p = v((ND+1)/2:end);
      end
      (1+gp)*Ipol - hbig == pos_trig_poly(nbig,1,Dcrt);
      hbig - (1-gp)*Ipol == pos_trig_poly(nbig,1,Dcrt);
      fci = fci + nDp(i);
    end
    
    fci = 0;
    for i = 1 : length(nDs) % stopband constraints
      clear Dcrt;
      for k = 1 : nDs(i)    % convert each polynomial in that constraint
        psize = size(Ds{fci+k});
        Dcrt{k}.d = (psize-1)/2;
        ND = prod(psize);
        v = vec(Ds{fci+k});
        Dcrt{k}.p = v((ND+1)/2:end);
      end
      gs*Ipol - hbig == pos_trig_poly(nbig,1,Dcrt);
      hbig + gs*Ipol == pos_trig_poly(nbig,1,Dcrt);
      fci = fci + nDs(i);
    end
    
    hbig(zpos) == 0;    % force extra coefficients to zero
cvx_end
hbig = mat([hbig(end:-1:2); hbig]);
h = hbig(nplus(1)+1:end-nplus(1), nplus(2)+1:end-nplus(2));
