function [H, err, E] = fir_deconv(G, D, Nx, crit, gamma, lincA, lincb)

% Find optimal solution to FIR deconvolution problem.
% Input:
%   G         - given channel model, represented as
%               G(z) = G{1} + G{2} z^{-1} + ...
%   D         - desired model
%   Nx        - order of FIR equalizer
%   crit      - optimization criterion:
%               0  - infinity norm
%               1  - quadratic norm
%               2  - mixed (minimize quadratic norm subject to infinity norm bound
%   gamma     - infinity norm bound
%   lincA
%   lincb     - linear constraints on H, in the form lincA * H = lincb
% Output:
%   H         - optimal filter
%   err       - error norm
%   E         - error FIR system

% BD 3.02.2006

[p,pg] = size(G{1});
[m,pd] = size(D{1});
if pg ~= pd
  error('Wrong sizes')
end
if p ~= pg || m ~= pd
  error('Only square systems allowd')
end
Nvar = (Nx+1) * m * p;

if nargin < 6 || isempty(lincA)  % no equality constraints
  Nrl = 0; Ncl = 0;
else
  % check equality constraints
  [Nrl,Ncl] = size(lincA);
  if Ncl ~= Nvar
    error('Wrong size lincA')
  end
  lincb = lincb(:);
  k = length(lincb);
  if k ~= Nrl
    error('Wrong size lincb')
  end
end

% augment model to account for channel noise
A{1} = [G{1} eye(pg)];
for i = 2 : length(G)
  A{i} = [G{i} zeros(pg)];
end
for i = 1 : length(D)
  B{i} = [D{i} zeros(pd)];
end

switch crit
  case 0,         % infinity norm optimization
    [H, E, err] = fit_infmin_left(A, B, Nx, lincA, lincb);
  case 1,         % quadratic optimization
    [H, E, err] = fit_2min_left(A, B, Nx, lincA, lincb);
  case 2,
    [H, E, err] = fit_2min_infb_left(A, B, Nx, gamma, lincA, lincb);
end
