function [h,Es,r] = fir_lowpass_minEs_mag(n, wp, ws, gp, gs)

% Design (nonlinear phase) FIR filter optimizing its
% stopband energy subject to magnitude constraints
% in passband and stopband.
% Input:
%   n       - filter order (must be even)
%   wp, ws  - passband and stopband edges (normalized frequencies)
%   gp, gs  - passband and stopband magnitude tolerances
% Output:
%   h       - filter coefficients
%   Es      - optimal stopband energy
%   r       - causal coefficients of R(z) = H(z) * H(1/z)

% BD 25.01.2006

nQ = n+1;     % length of polynomial
nS = n-1;
nQ2 = nQ * nQ;
nS2 = nS * nS;

Ks.f = nQ;    % the coefficients of the polynomial
Ks.s = [nQ nQ nQ nS nQ nS];
              % the matrices for (1)  R(w) >= 0
              %                  (2)  (1+gp)^2 - R(w) >= 0
              % (3,4) R(w) - (1-gp)^2 >= 0, w in [0,wp]
              % (5,6) gs^2 - R(w) >= 0,     w in [ws,pi]

nrA = 4*nQ;   % one LMI for each polynomial inequality
ncA = nQ + 4*nQ2 + 2*nS2;

As = sparse(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);

% the linear criterion
cs(1) = (1 - ws);
for k = 1 : n
  cs(k+1) = - 2*sin(pi*ws*k) / (pi*k);
end

% LMI 1: R(w) >= 0
InQ = spdiags( ones(nQ,1), 0, nQ, nQ );
rr = 1:nQ;    % row range in As for this LMI
As(rr, 1:nQ) = - InQ;
A1 = posintlmi_r(n, 0, 1);
As(rr, nQ+1:nQ+nQ2) = A1;

% LMI 2: (1+gp)^2 - R(w) >= 0
sc = nQ + nQ2;  % start column in As for this LMI matrices
rr = nQ+1:2*nQ;
As(rr, 1:nQ) = InQ;
As(rr, sc+1:sc+nQ2) = A1;
bs(nQ+1) = (1+gp)*(1+gp);

% LMI 3: R(w) - (1-gp)^2 >= 0, w in [0,wp]
sc = nQ + 2*nQ2;
rr = 2*nQ+1:3*nQ;
As(rr, 1:nQ) = - InQ;
[A1,B1] = posintlmi_r(n, 0, wp);
As(rr, sc+1:sc+nQ2) = A1;
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;
bs(2*nQ+1) = -(1-gp)*(1-gp);

% LMI 4: gs^2 - R(w) >= 0, w in [ws,pi]
sc = nQ + 3*nQ2 + nS2;
rr = 3*nQ+1:4*nQ;
As(rr, 1:nQ) = InQ;
[A1,B1] = posintlmi_r(n, ws, 1);
As(rr, sc+1:sc+nQ2) = A1;
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;
bs(3*nQ+1) = gs*gs;

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

r = x(1:nQ);      % coefficients of squared magnitude
Es = cs(1:nQ)' * r;

h = factsp_orch(r);
