function [h,Es] = fir_lowpass_minEs_mag_alin(n, tau, wp, ws, gp, gs)

% Design approximate linear-phase FIR filter optimizing its
% stopband energy subject to constraints in passband and stopband:
% | H(w) - exp(-tau*w) | <= gp, w in [0, wp]
% | H(w) | <= gs, w in [ws, pi]
%
% Implementation using BRL.
%
% Input:
%   n       - filter order (must be even)
%   tau     - ideal group delay
%   wp, ws  - passband and stopband edges (normalized frequencies)
%   gp, gs  - passband and stopband magnitude tolerances
% Output:
%   h       - filter coefficients
%   Es      - optimal stopband energy

% BD 26.01.2006

n1 = n+1;     % length of symmetric polynomial

nQ = n+2;     % size of first matrix in BRL
nS = n-1;     % size of second matrix
nQ2 = nQ * nQ;
nS2 = nS * nS;

Ks.f = n1;    % the coefficients of the polynomial
Ks.q = 1+n1;  % the stopband energy SOC
Ks.s = [nQ nS nQ nS]; % the matrices for:
              % (1,2) | H(w) - exp(-tau*w) | <= gp, w in [0, wp]
              % (3,4) | H(w) | <= gs, w in [ws, pi]

nrA = n1 + 2*(n1+nQ); % the SOC constraint + two LMIs for each norm inequality (BRL)
ncA = 1 + 2*n1 + 2*nQ2 + 2*nS2;

As = sparse(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(n1+1) = 1;    % minimize stopband energy

% the quadratic criterion
cc = zeros(n1,1);
cc(1) = (1 - ws);
for k = 1 : n
  cc(k+1) = - sin(pi*ws*k) / (pi*k);
end

% SOC
In1 = spdiags( ones(n1,1), 0, n1, n1 );
rr = 1:n1;    % row range in As for the SOC
As(rr, 1:n1) = real(sqrtm(toeplitz(cc)));  % square root needed in SOC
As(rr, n1+2:2*n1+1) = - In1;

% BRL 1: | H(w) - exp(-tau*w) | <= gp, w in [0, wp]
sc = 1+2*n1;      % start column in As for this BRL matrices
rr = n1+1:2*n1;   % row range in As for the first LMI
[A1,B1] = posintlmi_r(n, 0, wp);
for k = 1 : n1    % resize the first coefficient matrices
  A2 = sparse(nQ,nQ);
  A2(1:n1,1:n1) = mat( A1(k,:)' );
  As(n1+k, sc+1:sc+nQ2) = vec(A2)';
end
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;
bs(n1+1) = gp*gp;

% second LMI of BRL 1
sr = 2*n1+1;
for k = 1 : n1    % the last row of the matrix is h - e_tau
  A2 = sparse(nQ,nQ);
  A2(nQ,k) = 1;
  As(sr, sc+1:sc+nQ2) = vec(A2)';
  As(sr,k) = -1;
  sr = sr + 1;
end
bs(2*n1+1+tau) = -1;  % the group delay contribution
A2 = sparse(nQ,nQ);
A2(nQ,nQ) = 1;
As(sr, sc+1:sc+nQ2) = vec(A2)';
bs(sr) = 1;

% BRL 2: | H(w) | <= gs, w in [ws, pi]
sc = 1+2*n1+nQ2+nS2;    % start column in As for this BRL matrices
rr = 1+2*n1+nQ:3*n1+nQ; % row range in As for the first LMI
[A1,B1] = posintlmi_r(n, ws, 1);
As(rr, sc+1:sc+nQ2) = As(n1+1:2*n1,2+2*n1:1+2*n1+nQ2);
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;
bs(1+2*n1+nQ) = gs*gs;

% second LMI of BRL 2
sr = 3*n1+nQ+1;
for k = 1 : n1    % the last row of the matrix is h
  A2 = sparse(nQ,nQ);
  A2(nQ,k) = 1;
  As(sr, sc+1:sc+nQ2) = vec(A2)';
  As(sr,k) = -1;
  sr = sr + 1;
end
A2 = sparse(nQ,nQ);
A2(nQ,nQ) = 1;
As(sr, sc+1:sc+nQ2) = vec(A2)';
bs(sr) = 1;

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

h = x(1:n1);  % the filter
Es = x(n1+1)*x(n1+1);
