function [h,Es] = fir_lowpass_minEs_mag_alin_cvx(n, tau, wp, ws, gp, gs)

% Design approximate linear-phase FIR filter optimizing its
% stopband energy subject to constraints in passband and stopband:
%
% | H(w) - exp(-tau*w) | <= gp, w in [0, wp]
% | H(w) | <= gs, w in [ws, pi]
%
% Implementation using BRL, with CVX.
%
% Input:
%   n       - filter order (must be even)
%   tau     - ideal group delay
%   wp, ws  - passband and stopband edges (normalized frequencies)
%   gp, gs  - passband and stopband magnitude tolerances
% Output:
%   h       - filter coefficients
%   Es      - optimal stopband energy

% BD 7.05.2010

% the quadratic criterion
cc = zeros(n+1,1);
cc(1) = (1 - ws);
cc(2:end) = - sin(pi*ws*(1:n)') ./ (pi*(1:n)');
C = real(sqrtm(toeplitz(cc)));

del_vec = zeros(n+1,1);
del_vec(tau+1) = 1;   % vector representing z^{-tau}

cvx_begin
  variable h(n+1);
  variable vp(n+2);  % BRL variable for passband
  variable vs(n+2);  % BRL variable for stopband
  minimize norm(C*h);
  subject to
    vp == brl_trig_poly(n, 1, [0 wp*pi]);
    vp(1) == gp*gp;           % passband constraints
    vp(2:end) == h - del_vec;
    vs == brl_trig_poly(n, 1, [ws*pi pi]);
    vs(1) == gs*gs;           % stopband constraints
    vs(2:end) == h;
cvx_end
Es = h'*C'*C*h;
