function [h,Es] = fir_lowpass_minEs_mag_lin(n, wp, ws, gp, gs)

% Design linear-phase (symmetric) FIR filter optimizing its
% stopband energy subject to magnitude constraints
% in passband and stopband.
% Input:
%   n       - filter order (must be even)
%   wp, ws  - passband and stopband edges (normalized frequencies)
%   gp, gs  - passband and stopband magnitude tolerances
% Output:
%   h       - filter coefficients (symmetric vector)
%   Es      - optimal stopband energy

% BD 26.01.2006

if rem(n,2) == 1
  error('Only even order is possible');
end

nh = n/2;     % order of symmetric polynomial

nQ = nh+1;    % length of symmetric polynomial
nS = nh-1;
nQ2 = nQ * nQ;
nS2 = nS * nS;

Ks.f = nQ;    % the coefficients of the polynomial
Ks.q = 1+nQ;  % the stopband energy SOC
Ks.s = [nQ nQ nS nQ nS nQ nS]; % the matrices for:
              % (1)  1+gp-R(w) >= 0
              % (2,3) R(w)-1+gp >= 0, w in [0,wp] 
              % (4,5) gs-R(w) >= 0, w in [ws,pi]
              % (6,7) gs+R(w) >= 0, w in [ws,pi]

nrA = 5*nQ;   % the SOC constraint + one LMI for each polynomial inequality
ncA = 1 + 2*nQ + 4*nQ2 + 3*nS2;

As = sparse(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(nQ+1) = 1;    % minimize stopband energy

% the quadratic criterion
cc = zeros(n+1,1);
cc(1) = (1 - ws);
for k = 1 : n
  cc(k+1) = - sin(pi*ws*k) / (pi*k);
end
%P = zeros(n+1, nQ);
z = zeros(1,nQ);
z(end) = 1;
P = [ hankel(z); zeros(nh,1) eye(nh) ];
C = P' * toeplitz(cc) * P;  % quadratic form in distinct coefs

% SOC
InQ = spdiags( ones(nQ,1), 0, nQ, nQ );
rr = 1:nQ;    % row range in As for the SOC
As(rr, 1:nQ) = real(sqrtm(C));  % square root needed in SOC
As(rr, nQ+2:2*nQ+1) = - InQ;

% LMI 1: 1+gp-R(w) >= 0
sc = 1+2*nQ;      % start column in As for this LMI matrices
rr = nQ+1:2*nQ;   % row range in As for this LMI
A1 = posintlmi_r(nh, 0, 1);
As(rr, 1:nQ) = InQ;
As(rr, sc+1:sc+nQ2) = A1;
bs(nQ+1) = 1+gp;

% LMI 2: R(w)-1+gp >= 0, w in [0,wp]
sc = 1+2*nQ + nQ2;
rr = 2*nQ+1:3*nQ;
As(rr, 1:nQ) = - InQ;
[A1,B1] = posintlmi_r(nh, 0, wp);
As(rr, sc+1:sc+nQ2) = A1;
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;
bs(2*nQ+1) = -(1-gp);

% LMI 3: gs-R(w) >= 0, w in [ws,pi]
sc = 1+2*nQ + 2*nQ2 + nS2;
rr = 3*nQ+1:4*nQ;
As(rr, 1:nQ) = InQ;
[A1,B1] = posintlmi_r(nh, ws, pi);
As(rr, sc+1:sc+nQ2) = A1;
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;
bs(3*nQ+1) = gs;

% LMI 4: gs+R(w) >= 0, w in [ws,pi]
sc = 1+2*nQ + 3*nQ2 + 2*nS2;
rr = 4*nQ+1:5*nQ;
As(rr, 1:nQ) = - InQ;
As(rr, sc+1:sc+nQ2) = A1;
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;
bs(4*nQ+1) = gs;

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

h = [x(nQ:-1:2); x(1:nQ)];  % the symmetric filter
Es = x(nQ+1)*x(nQ+1);
