function [h,Es] = fir_lowpass_minEs_mag_lin_cvx(n, wp, ws, gp, gs)

% Design linear-phase (symmetric) FIR filter optimizing its
% stopband energy subject to magnitude constraints
% in passband and stopband.
% Implementation with CVX.
% Input:
%   n       - filter order (must be even)
%   wp, ws  - passband and stopband edges (normalized frequencies)
%   gp, gs  - passband and stopband magnitude tolerances
% Output:
%   h       - filter coefficients (symmetric vector)
%   Es      - optimal stopband energy

% BD 6.05.2010

if rem(n,2) == 1
  error('Only even order is possible');
end

nh = n/2;     % order of symmetric polynomial

% the quadratic criterion
cc = zeros(n+1,1);
cc(1) = (1 - ws);
for k = 1 : n
  cc(k+1) = - sin(pi*ws*k) / (pi*k);
end
z = zeros(1,nh+1);
z(end) = 1;
P = [ hankel(z); zeros(nh,1) eye(nh) ];
C = real(sqrtm(P' * toeplitz(cc) * P));  % matrix for SOC constraint

cvx_begin
  variable x(nh+1);   % filter coefficients (causal part)
  minimize norm(C*x);
  subject to
    (1 + gp) *eye(nh+1,1) - x == pos_trig_poly( nh, 1, [0 ws*pi] );
    (-1 + gp)*eye(nh+1,1) + x == pos_trig_poly( nh, 1, [0 wp*pi] );
    gs*eye(nh+1,1) - x == pos_trig_poly( nh, 1, [ws*pi pi] );
    gs*eye(nh+1,1) + x == pos_trig_poly( nh, 1, [ws*pi pi] );
cvx_end

h = [x(end:-1:2); x(1:end)];  % the symmetric filter
Es = x'*C'*C*x;
