function [X,err,E] = fit2_infmin_left(A, B, Nx, nplus)

% Find the FIR MIMO system H such that the infinity norm
% || XA-B || is minimized, where A and B are also FIR MIMO.
% 2 dimensional version
% Arguments:
%   A,B   - FIR MIMO systems, represented by cells:
%           A(z) = sum A{k1,k2} z1^{-k1} z2^{-k2}
%   Nx    - order of X
%   nplus - relaxation degree is Nx+nplus

% BD 7.02.2006

Nx = Nx + 1;      % size !
if nargin < 4
  nplus = [0 0];
end
Nt = Nx;          % true size
Nx = Nx + nplus;  % size for relaxation

% check arguments
Na = size(A);
Nb = size(B);

[pa,ma] = size(A{1,1});
[pb,mb] = size(B{1,1});
if ma ~= mb
  error('Wrong sizes')
end
px = pb;
mx = pa;

Sx = px*mx;
Sb = pb*mb;

% add zero coefficients to B(z), if too short
Ne = Na + Nx - 1;
for n1 = 1 : Ne(1)
  for n2 = 1 : Ne(2)
    if n1 > Nb(1) || n2 > Nb(2)
      B{n1,n2} = zeros(pb,mb);
    end
  end
end
Ng = Ne(1)*Ne(2)*pb + mb;       % size of extended Gram matrix

% number of zero constraints (if relaxation)
Nzc = (Nx(1)*Nx(2) - Nt(1)*Nt(2)) * Sx;

% vectorize XA and B
Neq = Ne(1) * Ne(2) * Sb;
Nvar = Nx(1) * Nx(2) * Sx;
Als = zeros(Neq,Nvar);
bls = zeros(Neq,1);

% the rows of the matrix
for ka1 = 1 : Na(1)     % put each block of A
  for ka2 = 1 : Na(2)
    blA = kron(A{ka1,ka2}', eye(px) );
    for kx1 = 1 : Nx(1)
      for kx2 = 1 : Nx(2)
        i1 = ka1 + kx1 - 1;
        i2 = ka2 + kx2 - 1;
        sr = (i2-1)*Ne(1) + i1-1;
        sc = (kx2-1)*Nx(1) + kx1-1;
        Als( sr*Sb+1 : (sr+1)*Sb, sc*Sx+1 : (sc+1)*Sx ) = blA;
      end
    end
  end
end

% the free term
ib = 1;
for n2 = 1 : Ne(2)
  for n1 = 1 : Ne(1)      % for each coefficient of the error
    bls(ib : ib+Sb-1) = vec( B{n1,n2} );
    ib = ib + Sb;
  end
end

Ks.f = Nvar;    % the elements of X
Ks.l = 1;       % the norm
Ks.s = Ng;

Nhp = ((2*Ne(1)-1) * (2*Ne(2)-1) + 1 ) / 2;
nrA = Neq + Nhp*Sb + mb*(mb+1)/2 + pb*(pb+1)/2 + Nzc;
ncA = 1 + Nvar + Ng*Ng;

As = zeros(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(Nvar+1) = 1;

% equalities for the BRL part
% the equalities for the coefficients of E
As(1:Neq, 1:Nvar) = Als;
bs(1:Neq) = bls;
ia = 1;       % row index in As
sc = Ng - mb;
for n2 = 1 : Ne(2)
  for n1 = 1 : Ne(1)  % for all matrix coefficients
    sr = ((n2-1)*Ne(1) + n1-1)*pb;
    for km = 1 : mb
      for kp = 1 : pb % for each element in the matrix
        Z = spalloc(Ng,Ng,1);
        Z( sr + kp, sc + km ) = 1;
        As(ia, Nvar+2:end) = vec(Z)';
        ia = ia+1;
      end
    end
  end
end

% the last diagonal block
for i = 1 : mb
  for k = i : mb
    Z = spalloc(Ng,Ng,1);
    Z( sc + i, sc + k) = 1;
    As(ia, Nvar+2:end) = vec(Z)';
    if i == k
      bs(ia) = 1;
    end
    ia = ia + 1;
  end
end

% the main diagonal block trace
for i = 1 : pb
  for k = i : pb
    Z = spalloc(Ng,Ng,Ne(1)*Ne(2));
    for z = 0 : Ne(1)*Ne(2)-1
      Z( z*pb + i, z*pb + k) = 1;
    end
    As(ia, Nvar+2:end) = vec(Z)';
    if i == k
      As(ia,Nvar+1) = -1;   % for the infinity norm
    end
    ia = ia + 1;
  end
end

% the other diagonal block traces are zero
dmax = max(Ne);
v1 = ones(dmax,1);
n1 = 1;
n2 = 0;
while n2 < Ne(2)
  T = kron( spdiags(v1, n2, Ne(2), Ne(2)), spdiags(v1, n1, Ne(1), Ne(1)) );
  Ts{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
  for i = 1 : pb
    for k = 1 : pb
      z = sparse(pb,pb,1);
      z(i,k) = 1;
      Z = sparse(Ng,Ng);
      Z(1:end-mb, 1:end-mb) = kron(T, z);
      As(ia, Nvar+2:end) = vec(Z)';
      ia = ia + 1;
    end
  end
  n1 = n1+1;
  if n1 == Ne(1)
    n1 = -Ne(1)+1;
    n2 = n2 + 1;
  end
end

% zero constraints on coefficients of X (if relaxation)
if Nzc
  for n2 = 1 : Nx(2)
    for n1 = 1 : Nx(1)
      if n1 > Nt(1) | n2 > Nt(2)
        sc = ((n2-1)*Nx(1) + n1-1)*Sx + 1;
        for i2 = 1 : mx
          for i1 = 1 : px
            As(ia, sc) = 1;
            ia = ia + 1;
            sc = sc + 1;
          end
        end
      end
    end
  end
end

%spy(As)
As = sparse(As);
%pars.fid = 0;
pars.eps = 1e-8;
[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info
  
err = sqrt(x(Nvar+1));

% form the big FIR solution (cutting the zeros, if relaxation)
i = 0;
for n2 = 1 : Nx(2)
  for n1 = 1 : Nx(1)
    if n1 <= Nt(1) && n2 <= Nt(2)
      X{n1,n2} = zeros(px,mx);
      for k = 1 : mx
        X{n1,n2}(:,k) = x(i+1 : i+px);
      end
    end
    i = i + px;
  end
end

% compute error (convolution)
Ev = Als * x(1 : Nx(1)*Nx(2)) - bls;
i = 0;
for n2 = 1 : Ne(2)
  for n1 = 1 : Ne(1)
    E{n1,n2} = zeros(pb,mb);
    for k = 1 : mb
      E{n1,n2}(:,k) = Ev(i+1 : i+pb);
    end
    i = i + pb;
  end
end
