function [X,E,en2] = fit_2min_infb_left(A, B, Nx, g, lincA, lincb)

% Find the FIR MIMO system X such that the 2-norm
% || XA-B || is minimized, where A and B are also FIR MIMO,
% subject to the constraint ||XA-B||_inf <= g.
% Arguments:
%   A,B   - FIR MIMO systems, represented by cells:
%           A(z) = A{1} + A{2} z^{-1} + ...
%   Nx    - order of X
%   g     - the bound for ||XA-B||_inf
%   lincA
%   lincb - linear constraints on X, in the form lincA * X = lincb

% BD 25.11.2004

Nx = Nx + 1;    % length
Na = length(A);
Nb = length(B);
[p,pa] = size(A{1});
[m,pb] = size(B{1});
if pa ~= pb
  error('Wrong sizes')
end
Sh = m*p;
Sb = m*pb;

% add zero coefficients to B(z), if too short
Ne = Na + Nx - 1;
for n = Nb+1 : Ne
  B{n} = zeros(m,pb);
end
Ng = Ne*pb + m;         % size of Gram matrix

% vectorize XA and B
Neq = Ne * m * pb;
Nvar = Nx * m * p;
Als = zeros(Neq,Nvar);
bls = zeros(Neq,1);

% the rows of the matrix
for ka = 1 : Na        % put each block of A
  blA = kron(A{ka}', eye(m) );
  for kh = 1 : Nx
    ia = ka+kh-1;
    Als( (ia-1)*Sb+1 : ia*Sb, (kh-1)*Sh+1 : kh*Sh ) = blA;
  end
end

% the free term
ib = 1;
for n = 1 : Ne        % for each coefficient of the error
  bls(ib : ib+Sb-1) = vec( B{n} );
  ib = ib + Sb;
end

if nargin < 5 || isempty(lincA)  % no equality constraints
  Nrl = 0; Ncl = 0;
else
  % check equality constraints
  [Nrl,Ncl] = size(lincA);
  if Ncl ~= Nvar
    error('Wrong size lincA')
  end
  lincb = lincb(:);
  k = length(lincb);
  if k ~= Nrl
    error('Wrong size lincb')
  end
end

Ks.f = Nvar;
Ks.q = 1 + Neq;
Ks.s = Ng;

nrA = Neq + Nrl + Neq + (Ne-1)*pb*pb + m*(m+1)/2 + pb*(pb+1)/2;
ncA = Nvar + 1 + Neq + Ng*Ng;

As = zeros(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(Nvar+1) = 1;

ncAg = Nvar + 1 + Neq;

% equality for the norm term
As(1:Neq, 1:Nvar) = Als;
As(1:Neq, Nvar+2:ncAg) = eye(Neq);
bs(1:Neq) = bls;
  
% the forced equality constraints
if Nrl
  As(Neq+1:Neq+Nrl, 1:Nvar) = lincA;
  bs(Neq+1:Neq+Nrl) = lincb;
end

% equalities for the BRL part
% the equalities for the coefficients of H
ia = Neq + Nrl + 1; % row index in As
for n = 1 : Ne      % for each matrix coefficient
  for i = 1 : m
    for k = 1 : pb   % for each element in the matrix
      Z = spalloc(Ng,Ng,1);
      Z( Ne*pb + i, (n-1)*pb + k ) = 1;
      As(ia, ncAg+1:end) = vec(Z)';
      %As(ia, (Nx-n)*m*p + (k-1)*m + i) = 1;
      As(ia, Nvar+1 + (Ne-n)*m*pb + (k-1)*m + i) = 1;
      ia = ia+1;
    end
  end
end

% the last diagonal block
for i = 1 : m
  for k = i : m
    Z = spalloc(Ng,Ng,1);
    Z( Ne*pb + i, Ne*pb + k) = 1;
    As(ia, ncAg+1:end) = vec(Z)';
    if i == k
      bs(ia) = 1;
    end
    ia = ia + 1;
  end
end

% the main diagonal block trace
for i = 1 : pb
  for k = i : pb
    Z = spalloc(Ng,Ng,Ne);
    for z = 0 : Ne-1
      Z( z*pb + i, z*pb + k) = 1;
    end
    As(ia, ncAg+1:end) = vec(Z)';
    if i == k
      bs(ia) = g*g;   % for the infinity norm
    end
    ia = ia + 1;
  end
end

% the other diagonal block traces are zero
for n = 1 : Ne-1
  for i = 1 : pb
    for k = 1 : pb
      Z = spalloc(Ng,Ng,Ne-n);
      for z = 0 : Ne-1-n
        Z( z*pb + i, (z+n)*pb + k) = 1;
      end
      As(ia, ncAg+1:end) = vec(Z)';
      ia = ia + 1;
    end
  end
end
%pars.fid = 0;
pars.eps = 1e-8;
[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info

en2 = x(Nvar+1);

% form the FIR solution
ix = 1;
for n = 1 : Nx
  X{n} = zeros(m,p);
  for k = 1 : p
    X{n}(:,k) = x(ix : ix+m-1);
    ix = ix + m;
  end
end

% compute error (convolution)
E = block_conv(X,A);
for n = 1 : Ne
  E{n} = E{n} - B{n};
end
