function [H,E,en2] = fit_2min_left(A, B, Nx, lincA, lincb)

% Find the FIR MIMO system H such that the 2-norm
% || XA-B || is minimized, where A and B are also FIR MIMO.
% The FIR MIMO systems are represented by cells:
% A(z) = A{1} + A{2} z^{-1} + ...
% Arguments:
%   A,B   - FIR MIMO systems, represented by cells:
%           A(z) = A{1} + A{2} z^{-1} + ...
%   Nx    - order of X
%   lincA
%   lincb - linear constraints on X, in the form lincA * X = lincb

% BD 23.11.2004

Nx = Nx + 1;        % length
Na = length(A);
Nb = length(B);
[p,pa] = size(A{1});
[m,pb] = size(B{1});
if pa ~= pb
  error('Wrong sizes')
end
Sh = m*p;
Sb = m*pb;

% add zero coefficients to B(z), if too short
Ne = Na + Nx - 1;
for n = Nb+1 : Ne
  B{n} = zeros(m,pb);
end

% form linear LS systems
Neq = Ne * m * pb;
Nvar = Nx * m * p;
Als = zeros(Neq,Nvar);
bls = zeros(Neq,1);

% the rows of the matrix
for ka = 1 : Na        % put each block of A
  blA = kron(A{ka}', eye(m) );
  for kh = 1 : Nx
    ia = ka+kh-1;
    Als( (ia-1)*Sb+1 : ia*Sb, (kh-1)*Sh+1 : kh*Sh ) = blA;
  end
end
%size(Als)
%spy(Als)

% the free term
ib = 1;
for n = 1 : Ne        % for each coefficient of the error
  bls(ib : ib+Sb-1) = vec( B{n} );
  ib = ib + Sb;
end

if nargin < 4 || isempty(lincA) % no equality constraints ==> solve LS system
  x = Als \ bls;
else                  % optimization
  % check equality constraints
  [Nrl,Ncl] = size(lincA);
  if Ncl ~= Nvar
    error('Wrong size lincA')
  end
  lincb = lincb(:);
  k = length(lincb);
  if k ~= Nrl
    error('Wrong size lincb')
  end

  Ks.f = Nvar;
  Ks.q = 1 + Neq;
  nrA = Neq + Nrl;
  ncA = Nvar + 1 + Neq;

  As = zeros(nrA,ncA);
  bs = zeros(nrA,1);
  cs = zeros(ncA,1);
  cs(Nvar+1) = 1;

  % equality for the norm term
  As(1:Neq, 1:Nvar) = Als;
  As(1:Neq, Nvar+2:end) = eye(Neq);
  bs(1:Neq) = bls;
  
  % the forced equality constraints
  As(Neq+1:end, 1:Nvar) = lincA;
  bs(Neq+1:end) = lincb;
  
  %pars.fid = 0;
  pars.eps = 1e-8;
  [x,y,info] = sedumi(As,bs,cs,Ks,pars);
  info
  
  err_min = x(Nvar+1);
end

% form the FIR solution
ix = 1;
for n = 1 : Nx
  H{n} = zeros(m,p);
  for k = 1 : p
    H{n}(:,k) = x(ix : ix+m-1);
    ix = ix + m;
  end
end

% compute error (convolution)
E = block_conv(H,A);
for n = 1 : Ne
  E{n} = E{n} - B{n};
end
en2 = norm_fir_2(E);
