function [X,E,err,As,bs] = fit_infmin_left(A, B, Nx, lincA, lincb)

% Find the FIR MIMO system X such that the infinity norm
% || XA-B || is minimized, where A and B are also FIR MIMO.
% Arguments:
%   A,B   - FIR MIMO systems, represented by cells:
%           A(z) = A{1} + A{2} z^{-1} + ...
%   Nx    - order of X
%   lincA
%   lincb - linear constraints on X, in the form lincA * X = lincb

% BD 26.11.2004

% check arguments
Nx = Nx + 1;      % length
Na = length(A);
Nb = length(B);
[p,pa] = size(A{1});
[m,pb] = size(B{1});
if pa ~= pb
  error('Wrong sizes')
end
Sh = m*p;
Sb = m*pb;

% add zero coefficients to B(z), if too short
Ne = Na + Nx - 1;
for n = Nb+1 : Ne
  B{n} = zeros(m,pb);
end
Ng = Ne*pb + m;         % size of Gram matrix

% vectorize XA and B
Neq = Ne * m * pb;
Nvar = Nx * m * p;
Als = zeros(Neq,Nvar);
bls = zeros(Neq,1);

% the rows of the matrix
for ka = 1 : Na        % put each block of A
  blA = kron(A{ka}', eye(m) );
  for kh = 1 : Nx
    ia = ka+kh-1;
    Als( (ia-1)*Sb+1 : ia*Sb, (kh-1)*Sh+1 : kh*Sh ) = blA;
  end
end

% the free term
ib = 1;
for n = 1 : Ne        % for each coefficient of the error
  bls(ib : ib+Sb-1) = vec( B{n} );
  ib = ib + Sb;
end

if nargin < 4 || isempty(lincA)  % no equality constraints
  Nrl = 0; Ncl = 0;
else
  % check equality constraints
  [Nrl,Ncl] = size(lincA);
  if Ncl ~= Nvar
    error('Wrong size lincA')
  end
  lincb = lincb(:);
  k = length(lincb);
  if k ~= Nrl
    error('Wrong size lincb')
  end
end

Ks.f = Nvar;    % the elements of X
Ks.l = 1;       % the norm
Ks.s = Ng;

nrA = Nrl + Neq + (Ne-1)*pb*pb + m*(m+1)/2 + pb*(pb+1)/2;
ncA = 1 + Nvar + Ng*Ng;

As = zeros(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(Nvar+1) = 1;

% the forced equality constraints
if Nrl
  As(1:Nrl, 1:Nvar) = lincA;
  bs(1:Nrl) = lincb;
end

% equalities for the BRL part
% the equalities for the coefficients of E
As(Nrl+1:Nrl+Neq, 1:Nvar) = Als;
bs(Nrl+1:Nrl+Neq) = bls;
ia = Nrl + 1; % row index in As
for n = Ne : -1 : 1  % for each matrix coefficient, in the order
  for k = 1 : pb   % for each element in the matrix
    for i = 1 : m
      Z = spalloc(Ng,Ng,1);
      Z( Ne*pb + i, (n-1)*pb + k ) = 1;
      As(ia, Nvar+2:end) = vec(Z)';
      %bs(ia) = -H{N+1-n}(i,k);
      ia = ia+1;
    end
  end
end

% the last diagonal block
for i = 1 : m
  for k = i : m
    Z = spalloc(Ng,Ng,1);
    Z( Ne*pb + i, Ne*pb + k) = 1;
    As(ia, Nvar+2:end) = vec(Z)';
    if i == k
      bs(ia) = 1;
    end
    ia = ia + 1;
  end
end

% the main diagonal block trace
for i = 1 : pb
  for k = i : pb
    Z = spalloc(Ng,Ng,Ne);
    for z = 0 : Ne-1
      Z( z*pb + i, z*pb + k) = 1;
    end
    As(ia, Nvar+2:end) = vec(Z)';
    if i == k
      As(ia,Nvar+1) = -1;   % for the infinity norm
    end
    ia = ia + 1;
  end
end

% the other diagonal block traces are zero
for n = 1 : Ne-1
  for i = 1 : pb
    for k = 1 : pb
      Z = spalloc(Ng,Ng,Ne-n);
      for z = 0 : Ne-1-n
        Z( z*pb + i, (z+n)*pb + k) = 1;
      end
      As(ia, Nvar+2:end) = vec(Z)';
      ia = ia + 1;
    end
  end
end

%spy(As)

As = sparse(As);
%pars.fid = 0;
pars.eps = 1e-8;
[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info
  
err = sqrt(x(Nvar+1));

% form the FIR solution
ix = 1;
for n = 1 : Nx
  X{n} = zeros(m,p);
  for k = 1 : p
    X{n}(:,k) = x(ix : ix+m-1);
    ix = ix + m;
  end
end

% compute error (convolution)
E = block_conv(X,A);
for n = 1 : Ne
  E{n} = E{n} - B{n};
end
