function [X,err,E] = fit_infmin_rob_left(A, B, Nx)

% Find the FIR MIMO system X such that the infinity norm
% || XA-B || is minimized, where A and B are also FIR MIMO.
% A belong to a polytope whose vertices are known.
% Arguments:
%   A     - unknown FIR MIMO system, bounded by a polytope, represented by cells:
%           A(z) = A{i,1} + A{i,2} z^{-1} + ...
%           with i = 1 : I, where I is the number of vertices of the polytope
%   B     - FIR system (known), similar cell description
%   Nx    - order of X
%   lincA
%   lincb - linear constraints on X, in the form lincA * X = lincb

% BD 9.02.2006

% check arguments
Nx = Nx + 1;      % length
Ip = length(A);   % number of vertices in the polytope
Na = length(A{1});
Nb = length(B);
[p,pa] = size(A{1}{1});
[m,pb] = size(B{1});
if pa ~= pb
  error('Wrong sizes')
end
Sh = m*p;
Sb = m*pb;

% add zero coefficients to B(z), if too short
Ne = Na + Nx - 1;
for n = Nb+1 : Ne
  B{n} = zeros(m,pb);
end
Ng = Ne*pb + m;         % size of Gram matrix
Ng2 = Ng*Ng;

% vectorize B
Neq = Ne * m * pb;
Nvar = Nx * m * p;
bls = zeros(Neq,1);
ib = 1;
for n = 1 : Ne        % for each coefficient of the error
  bls(ib : ib+Sb-1) = vec( B{n} );
  ib = ib + Sb;
end

Ks.f = Nvar;    % the elements of X
Ks.l = 1;       % the norm
Ks.s = Ng* ones(1,Ip);

nrA = (Neq + (Ne-1)*pb*pb + m*(m+1)/2 + pb*(pb+1)/2) * Ip;
ncA = 1 + Nvar + Ng2*Ip;

As = zeros(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(Nvar+1) = 1;

ia = 1;           % row index in As
for ip = 1 : Ip   % put all conditions for each vertex of the polytope
  
  % vectorize XA and build matrix Als such that Als * vec(X) = vec(B)
  Als = zeros(Neq,Nvar);
  for ka = 1 : Na        % put each block of A
    blA = kron(A{ip}{ka}', eye(m) );
    for kh = 1 : Nx
      ii = ka+kh-1;
      Als( (ii-1)*Sb+1 : ii*Sb, (kh-1)*Sh+1 : kh*Sh ) = blA;
    end
  end

  % equalities for the BRL part
  sc = Nvar+1 + (ip-1)*Ng2;   % start column for current matrix variable
  % the equalities for the coefficients of E
  As(ia:ia+Neq-1, 1:Nvar) = Als;
  bs(ia:ia+Neq-1) = bls;
  for n = 1 : Ne      % for each matrix coefficient
    for k = 1 : pb    % for each element in the matrix
      for i = 1 : m
        Z = spalloc(Ng,Ng,1);
        Z( Ne*pb + i, (n-1)*pb + k ) = 1;
        As(ia, sc+1:sc+Ng2) = vec(Z)';
        ia = ia+1;
      end
    end
  end

  % the last diagonal block
  for i = 1 : m
    for k = i : m
      Z = spalloc(Ng,Ng,1);
      Z( Ne*pb + i, Ne*pb + k) = 1;
      As(ia, sc+1:sc+Ng2) = vec(Z)';
      if i == k
        bs(ia) = 1;
      end
      ia = ia + 1;
    end
  end

  % the main diagonal block trace
  for i = 1 : pb
    for k = i : pb
      Z = spalloc(Ng,Ng,Ne);
      for z = 0 : Ne-1
        Z( z*pb + i, z*pb + k) = 1;
      end
      As(ia, sc+1:sc+Ng2) = vec(Z)';
      if i == k
        As(ia,Nvar+1) = -1;   % for the infinity norm
      end
      ia = ia + 1;
    end
  end

  % the other diagonal block traces are zero
  for n = 1 : Ne-1
    for i = 1 : pb
      for k = 1 : pb
        Z = spalloc(Ng,Ng,Ne-n);
        for z = 0 : Ne-1-n
          Z( z*pb + i, (z+n)*pb + k) = 1;
        end
        As(ia, sc+1:sc+Ng2) = vec(Z)';
        ia = ia + 1;
      end
    end
  end
end

As = sparse(As);
%pars.fid = 0;
pars.eps = 1e-8;
[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info
  
err = sqrt(x(Nvar+1));

% form the FIR solution
ix = 1;
for n = 1 : Nx
  X{n} = zeros(m,p);
  for k = 1 : p
    X{n}(:,k) = x(ix : ix+m-1);
    ix = ix + m;
  end
end

% compute error (convolution) for vertices
for ip = 1 : Ip
  Ei = block_conv(X,A{ip});
  for n = 1 : Ne
    E{ip}{n} = Ei{n} - B{n};
  end
end
