function [hp, Es, g] = gdft_orth(M, Nh, L, tolort, gs)

% Compute real prototype of orthogonal GDFT-modulated NPR filterbank.
% Method: LS optimization of M-band filter, then spectral factorization.
% Input:
%   M       - number of channels (to cover [0, 2*pi])
%   N       - filter order
%   L       - downsampling factor 
%   tolort  - tolerance to orthogonality constraint (implicit value 0.001)
%   gs      - stopband error bound (stopband is [pi/L,pi]) (ignored, if >= 1)
% Return:
%   hp      - prototype filter
%   Es      - stopband energy of the filter (LS error in [pi/L,pi] minimized)
%   g       - product filter

% BD 13.07.2006, 10.05.2010

if nargin < 5
  gs = 1;   % not actually used
end
if nargin < 4 || isempty(tolort)
  tolort = 0.001;
end

P = floor(Nh/M); % number of coefficients that enter the PR constraint
Nh = Nh+1;       % filter length
ws = 1 / L;

% the linear criterion
cc = zeros(Nh,1);
cc(1) = 1 - ws;
for k = 1 : Nh-1
  cc(k+1) = - sin(pi*ws*k) / (pi*k);
end

if P == 0     % short prototype, no Nyquist condition necessary
  [v,d] = eig(toeplitz(cc));    %  take eigenvector of criterion matrix
  [dm,im] = min(diag(d));
  hp = v(:,im);
  hp = hp / norm(hp) * sqrt(L/M);
  return
end

% Sedumi parameters
if gs >= 1  % if no stopband constraint
  nrA = P+2;        % number of equality constraints (!!!one variable = 0 !!!)
  ncA = P+1+Nh*Nh;  % number of variables
  K.s = Nh;
else
  nrA = P+2+Nh;     % +Nh for the stopband constraint
  ncA = P+1+2*Nh*Nh+(Nh-2)*(Nh-2);  % + two matrices for the same
  K.s = [Nh Nh Nh-2];
end
K.q = P+1;

As = zeros( nrA, ncA );
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );

colQ = P+2 : P+1+Nh*Nh;
cs(colQ) = vec(toeplitz(cc)); 	% symmetric Toeplitz matrix - the criterion
bs(1) = L/M;                    % r_0 = L/M (normalization)

e = ones(Nh,1);
for i = 1:P+1
  As(i,colQ) = vec( diag(e(1:Nh-(i-1)*M), M*(i-1)) )';
end

As(2:P+1,2:P+1) = eye(P);       % slack variable
As(P+2,1) = 1;
bs(P+2) = sqrt(tolort);

if gs < 1   % stoband error constraint
  [A1,A2] = posintlmi_r(Nh-1, 1/L, 1);     % matrices for the stopband positivity constraint
  As(P+3:end, P+2+Nh*Nh:end) = [A1 A2];
  for i = 1 : Nh  % traces of Q for the coefficients of the product filter
    As(P+2+i, colQ) = vec( diag(e(1:Nh-(i-1)), (i-1)) )';
  end
  bs(P+3) = gs*gs;
end

pars.fid = 0;
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,K,pars);
info

X = mat(x(colQ));
g = zeros(Nh,1);
for i=1:Nh
  g(i) = sum(diag(X,i-1));
end
hp = factsp_orch(g);

Es = hp'*toeplitz(cc)*hp;
