function [a,b,cost,iter] = iir2_gn_prstab(a, b, rho, inda, freq1, freq2, ...
                                         Hid, weight, max_count, tol)
% Design an 2D IIR filter, using the Gauss-Newton (Lang) method.
% The stability domain is described by positive realness.
% Input:
%   a             - initial denominator
%   b             - initial numerator
%   rho           - max pole radius
%   inda          - what type of 2D filter (quarter plane=0, half plane>0)
% freq1, freq2  - frequency mesh, L1xL2 points
% Hid            - desired frequency response, on the mesh
% weight        - the weight associated with the frequency points
% max_count     - maximum number of iterations
% tol           - relative tolerance for stopping the iterations

% BD 20.02.2002

L1 = length(freq1);
L2 = length(freq2);
if [L1,L2] ~= size(Hid)
   error('Incorrect desired response vector length')
end
if [L1,L2] ~= size(weight)
   error('Incorrect weights vector length')
end

if nargin < 9
  max_count = 50;
end
if nargin < 10
  tol = 1e-5;
end

freq1 = freq1(:);
freq2 = freq2(:);
rho = rho(1);

t = cputime;

[n1,n2] = size(a);
[m1,m2] = size(b);
fprintf('2D Gauss-Newton      m1=%d  m2=%d  n1=%d  n2=%d  rho=%g\n', ...
        m1-1, m2-1, n1-1, n2-1, rho(1));

% Matrices with exponents

Exp_n1 = exp( -j*freq1(:)*(0:n1-1) );
Exp_n2 = exp( -j*freq2(:)*(0:n2-1) );
Exp_m1 = exp( -j*freq1(:)*(0:m1-1) );
Exp_m2 = exp( -j*freq2(:)*(0:m2-1) );
Ca = kron( Exp_n2, Exp_n1 );
Ca(:,1) = [];                   % a00=1 !
Cb = kron( Exp_m2, Exp_m1 );

% Parameters for the SDP problem that do not change

nva = n1*n2 - 1;                % number of variables in A
nvb = m1*m2;                    % number of variables in B
nvab = nva + nvb;
N = n1 * n2;
M = N + (n1-1)*(n2-1);          % number of "gen trace" constraints
r1 = (n2-1)*(2*n1-1) + n1;      % first useful row of convolution matrix

nrA = M + nvab;                 % number of equality constraints
ncA = nvab + 1 + nvab + N*N;    % number of variables

K.l = nvab;
K.q = 1 + nvab;
K.s = N;

As = zeros( nrA, ncA );
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );
cs(nvab+1) = 1;

ect = 1000*ones(nvab,1);        % constant term to get positive variable

% generalized "trace" equalities
k1=0 ; k2=0;
k = 1;
while k1 < n1 && k2 < n2
  As(k, 2*nvab+2:end) = ...
      vec(kron(diag(ones(n2-k2,1),-k2), diag(ones(n1-abs(k1),1),-k1)))';
  k1 = k1 + 1;
  if k1 == n1
    k1 = -n1+1;
    k2 = k2+1;
  end
  k = k+1;
end

% "quadratic" equalities
As(M+1 : end, nvab+2 : 2*nvab+1) = eye(nvab);

pars.fid = 0;
pars.eps = 1e-10;

% Initial values for the algorithm

cost_old = J_2d(a, b, freq1, freq2, Hid, weight);
disp([ 0, cost_old ] )

rhov1 = rho*ones(n1,1);
rhov1(1) = 1;
rhov1 = cumprod(rhov1);
rhov2 = rho*ones(n2,1);
rhov2(1) = 1;
rhov2 = rhov1(end) * cumprod(rhov2);
R = hankel(rhov1,rhov2);
iR = 1 ./ vec(R);

err = 1;
iter = 0;
while (err > tol) && (iter < max_count)

  % compute gradient of H(a,b) and associated quadratic criterion

  Aw = polyval_2d(a, exp(-j*freq1), exp(-j*freq2));
  Bw = polyval_2d(b, exp(-j*freq1), exp(-j*freq2));
  Aw = Aw(:);
  Bw = Bw(:);
  Hw = Bw ./ Aw;

  v = weight(:) .* ( Hid(:) - Hw );
  C = [repmat(-Hw./Aw,1,size(Ca,2)).*Ca, repmat(1./Aw,1,size(Cb,2)).*Cb];
  Cw = repmat( weight(:), 1, size(C,2) ) .* C;
 
  p_crit = -2*real(C'*v);
  %S_crit = real(C'*Cw) + 1e-5*eye(nvab);
  S_crit = real(C)'*real(Cw) + imag(C)'*imag(Cw) + 1e-5*eye(nvab);
   
  arho = a ./ R;
  
  % build associated matrices
  Ls= chol(S_crit);

  aa = reshape( fliplr(arho(:)')', n1, n2 );
  F = convmtx2( aa, n1, n2 );   % block toeplitz, with reversed columns
  F = F( r1:end, :);            % keep only significant rows

  %G = convmtx2( a, n1, n2 );
  G = convmtx2( arho, n1, n2 );
  G = G( r1:end, : );
  G = G(:,end:-1:1);            % block Hankel with Hankel blocks !
  %F = F + G;
  F = (F + G) * diag(iR);
  F(:,1) = [];                  % delete column of a00

  %f = 2 * G * a(:);
  f = 2 * G * arho(:);
  yc = 0.5 * (Ls' \ p_crit);

  % formulate the SDP problem (or what is left to be formulated)

  % generalized "trace" equalities
  As(1:M, 1:nva) = -F;
  bs(1:M) = f - F * ect(1:nva);
 
  % "quadratic" equalities
  As(M+1 : end, 1:nvab) = -Ls;
  bs(M+1 : end) = yc - Ls*ect;

  [x,y,info] = sedumi(As,bs,cs,K,pars);
 
  % result: maximum step on a and b
  delta_a = [ 0; x(1:nva) - ect(1:nva) ];
  delta_a = reshape( delta_a, n1, n2 );
  delta_b = x(nva+1:nvab) - ect(nva+1:nvab);
  delta_b = reshape( delta_b, m1, m2 );
  alpha_in = 0.6*max_alpha_2d(delta_a, a, rho, inda);

  [alpha, cost] = min_gold_2d(a, b, freq1, freq2, Hid, weight,...
				 delta_a, delta_b, alpha_in);

  if  stab2wlb( (a + alpha*delta_a) ./ R )
    cost = cost_old;
    disp('Roots outside the circle !!!')
    break
  end

  a = a + alpha*delta_a;
  b = iir2_ls_fixa( m1-1, m2-1, a, freq1, freq2, Hid, weight);
  cost = J_2d(a, b, freq1, freq2, Hid, weight);

  err = (cost_old - cost)/cost_old;
   
  iter = iter + 1;
  %disp( [iter, cost, err, alpha/alpha_in*100, cputime-t] )
  stab_rad = 1/stab_marg_wlb2(a,2,1e-5);
  fprintf('it=%d  cost=%g  errel=%g  alf=%5.2f  r=%4.3g  t=%5.1f\n', ...
            iter, cost, err, alpha/alpha_in*100, stab_rad, cputime-t)
  cost_old = cost;
end
