function [a,b,cost,iter] = iir_gn_prstab(n, m, rho, freq, dH, weight, a, b, max_count)
%
% Design an IIR filter using the Gauss-Newton method,
% with stability domain described by positive realness
%
% Input:
%   n		        - denominator degree
%   m	      	  - numerator degree
%   rho		      - radius of circle where the roots must hold
%   freq	      - frequency grid, L points
%   dH		      - desired frequency response, on the grid
%   weight	    - the weight associated with the frequency points
%   a,b		      - initialization (optional)
%   max_count   - maximum number of iterations (implicit 20)
% Output:
%   a,b         - optimized denominator and numerator
%   cost        - criterion value
%   iter        - number of iterations actually performed

% BD 22.10.2001
% RN 14.02.2001
% BD 28.07.2006 (revised)

fprintf('\nGauss-Newton\n')

L = length(freq);
if L ~= length(dH)
   error('Incorrect desired response vector length')
end
if L ~= length(weight)
   error('Incorrect weights vector length')
end

t = cputime;

m1 = m+1;
n1 = n+1;
nm1 = n+m+1;

if nargin < 9
  max_count = 40;
end
if nargin < 7
  a = zeros(1,n);
  b = eye(1,m1);
else
  b = b(:)' / a(1);
  a = a(:)' / a(1);
  a = a(2:end);       % ignore first coefficient (=1)
end

% Matrices with exponents

Exp_n=exp( -j*freq(:)* (1:n));
Exp_m=exp( -j*freq(:)* (0:m));

% Parameters for the SDP problem that do not change

nrA = n1 + nm1;			% number of equality constraints
ncA = nm1 + 1 + nm1 + n1*n1;		% number of variables
   
K.l = nm1;
K.q = 1 + nm1;
K.s = n1;
   
As = zeros( nrA, ncA );
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );
cs(nm1+1) = 1;
   
ect = 1000*ones(nm1,1);	% constant term to get positive variable
   
% "trace" equalities
for i = 1 : n1
  v = zeros(n1,1);
  v(i) = 1;
  As(i, 2*nm1+2 : end) = vec(tril(toeplitz( v)))';
end
   
% "quadratic" equalities
As(n1+1 : end, nm1+2 : 2*nm1+1) = eye(nm1);
   
pars.fid = 0;
pars.eps = 1e-10;

% Initial values for the algorithm

cost_old = J2(weight, dH, freq, b, [1 a]);

disp([ 0, cost_old ] )

R = diag(cumprod(rho*ones(n,1)));
iR = diag( 1./ diag(R) );

err = 1;
tol = 1e-5;
iter = 0;
while (err > tol) & (iter < max_count)

  % compute gradient of H(a,b) and associated quadratic criterion
  Bw=zpolyval(b,exp(j*freq(:)));
  Aw=zpolyval([1,a],exp(j*freq(:)));
  
  v=(dH(:)-Bw./Aw);
  M=[repmat(-Bw./(Aw.^2),1,n).*Exp_n, repmat(1./Aw,1,m+1).*Exp_m];
   
  c_crit=real(v'*diag(weight)*v);
  p_crit=-2*real(M'*diag(weight)*v);
  S_crit=real(M'*diag(weight)*M) + 1e-5*eye(nm1);
   
  arho = a./(diag(R)');
  
  % build associated matrices
  Ls= chol(S_crit);
  F = toeplitz( [arho(1) 1 zeros(1, n-1)], arho ) +...
      hankel( [arho 0], zeros(1,n) );
  F = F / R;
  f = 2 * triu(toeplitz([1 arho])) * [1; arho'];
  yc = 0.5 * (Ls' \ p_crit);
   
  % formulate the SDP problem (or what is left to be formulated)

  % "trace" equalities
  As(1:n1, 1:nm1) = -[F zeros(n1,m1)];
  bs(1:n1) = f - [F zeros(n1,m1)] * ect;
   
  % "quadratic" equalities
  As(n1+1 : end, 1:nm1) = -Ls;
  bs(n1+1 : end) = yc - Ls*ect;
  
  [x,y,info] = sedumi(As,bs,cs,K,pars);
   
  % result: maximum step on a and b
  delta_a = (x(1:n) - ect(1:n))';
  delta_b = (x(n1:nm1) - ect(n1:nm1))';
  alpha_in = 0.9*max_alpha(delta_a, a, rho); 
   
  [alpha, cost] = min_gold(weight, dH, freq, b, a, delta_b, delta_a, alpha_in);

  max_root = max(abs(roots([1 a+alpha*delta_a])));
  if  max_root > rho
    cost = cost_old;
    %disp('Roots outside the circle !!!')
    break
  end

  a = a + alpha*delta_a;
  b = iir_ls_fixa( m, [1 a], freq, dH, weight);
  cost = J2(weight, dH, freq, b, [1 a]);
  err = (cost_old - cost)/cost_old;
   
  iter = iter + 1;
  disp( [iter, cost, err, alpha/alpha_in*100, cputime-t] )
  cost_old = cost;
end
a = [1 a];
