function [b,a] = iir_lowpass_mag(m, n, wp, ws, gp, gs, dstaba)

% Minimax optimization of lowpass IIR filter with magnitude
% constraints in passband and stopband.
% Input:
%   m, n        - degrees of numerator, denominator
%   wp, ws      - passband and stopband edges (normalized frequencies)
%   gp, gs      - passband and stopband error bounds
%   dstaba      - constant added to Ra(z) to increase stability margin
% Output:
%   b, a        - numerator and denominator coefficients

% BD 24.07.2006

if nargin < 7
  dstaba = 0;
end

% lengths and sizes
nB = m+1;
nA = n+1;     % length of polynomials
nQ = max(nB,nA);
nS = nQ-2;
nAB = nA + nB;
nA2 = nA * nA;
nB2 = nB * nB;
nQ2 = nQ * nQ;
nS2 = nS * nS;

% Sedumi parameters
Ks.f = nB + nA;   % the coefficients of (squared) numerator, denominator
Ks.s = [nB nA nQ nQ nS nQ nS];
              % the matrices for (1)  Rb(w) >= 0
              %                  (2)  Ra(w) >= 0
              %                  (3)  (1+gp)^2 Ra(w) - Rb(w) >= 0
              % (4,5) Rb(w)-(1-gp)^2 Ra(w) >= 0, w in [0,wp]
              % (6,7) gs^2 Ra(w) - Rb(w) >= 0,   w in [ws,pi]

nrA = 1 + nB + nA + 3*nQ;   % ra0 = 1 and one LMI for each polynomial inequality
ncA = nB + nA + 3*nQ2 + 2*nS2;

As = sparse(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);    % feasibility problem !

% LMI 1: Rb(w) >= 0
InB = spdiags( ones(nB,1), 0, nB, nB );
rr = 1:nB;    % row range in As for this LMI
As(rr, 1:nB) = - InB;
A1 = posintlmi_r(m, 0, 1);
As(rr, nAB+1:nAB+nB2) = A1;

% LMI 2: Ra(w) - dstaba >= 0
InA = spdiags( ones(nA,1), 0, nA, nA );
rr = nB+1:nAB;    % row range in As for this LMI
sc = nAB+nB2;     % start column in As for matrix variable
As(rr, nB+1:nAB) = - InA;
A1 = posintlmi_r(n, 0, 1);
As(rr, sc+1:sc+nA2) = A1;
bs(nB+1) = -dstaba;

% LMI 3: (1+gp)^2 Ra(w) - Rb(w) >= 0
InQ = spdiags( ones(nQ,1), 0, nQ, nQ );
sr = nAB;
rr = sr+1 : sr+nQ;
sc = nAB + nB2 + nA2;  % start column in As for this LMI matrices
As(sr+1:sr+nB, 1:nB) = InB;
As(sr+1:sr+nA, nB+1:nAB) = - (1+gp)*(1+gp) * InA;
A1 = posintlmi_r(nQ-1, 0, 1);
As(rr, sc+1:sc+nQ2) = A1;

% LMI 4: Rb(w)-(1-gp)^2 Ra(w) >= 0, w in [0,wp]
sr = nAB + nQ;
rr = sr+1 : sr+nQ;
sc = nAB + nB2 + nA2 + nQ2;
As(sr+1:sr+nB, 1:nB) = - InB;
As(sr+1:sr+nA, nB+1:nAB) = (1-gp)*(1-gp) * InA;
[A1,B1] = posintlmi_r(nQ-1, 0, wp);
As(rr, sc+1:sc+nQ2) = A1;
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;

% LMI 5: gs^2 Ra(w) - Rb(w) >= 0, w in [ws,pi]
sr = nAB + 2*nQ;
rr = sr+1 : sr+nQ;
sc = nAB + nB2 + nA2 + 2*nQ2 + nS2;
As(sr+1:sr+nB, 1:nB) = InB;
As(sr+1:sr+nA, nB+1:nAB) = - gs*gs * InA;
[A1,B1] = posintlmi_r(nQ-1, ws, 1);
As(rr, sc+1:sc+nQ2) = A1;
As(rr, sc+nQ2+1:sc+nQ2+nS2) = B1;

% the norming constraint ra0 = 1
As(end, nB+1) = 1;
bs(end) = 1;

%spy(As)

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

rb = x(1:nB);      % coefficients of numerator squared magnitude
b = factsp_orch(rb);
ra = x(nB+1:nAB);  % coefficients of denominator squared magnitude
a = factsp_orch(ra);
