function [b, gs] = iir_lowpass_minmax_alin_fixa(m, a, tau, wp, ws, gp, gs)

% Design approximate linear-phase IIR filter with given denominator
% optimizing its stopband error gs subject to constraints in passband:
% | H(w) - exp(-tau*w) | <= gp, w in [0, wp]
% | H(w) | <= gs, w in [ws, pi]
% Input:
%   m           - degree of numerator
%   a           - denominator
%   tau         - group delay
%   wp, ws      - passband and stopband edges (normalized frequencies)
%   gp          - passband error bound
%   gs          - if given, it's the stopband error bound
%                 (feasibility problem solved !)
% Output:
%   b           - numerator coefficients
%
% Implementation using BRL.

% BD 26.07.2006

if nargin < 7
  gs = 0;
end

n1 = length(a);
n = n1 - 1;   % denominator degree
if n > m
  error('Denominator degree should not be greater than numerator degree')
end

N = max( m, n + tau );  % max degree possible
N1 = N+1;     % length of symmetric polynomial
m1 = m + 1;

NQ = N + 2;
NS = N - 1;   % sizes of matrices in first BRL
NQ2 = NQ * NQ;
NS2 = NS * NS;

mQ = m + 2;
mS = m - 1;   % sizes of matrices in second BRL
mQ2 = mQ * mQ;
mS2 = mS * mS;

Ks.f = 1 + m1;  % gs and the coefficients of the polynomial
Ks.s = [NQ NS mQ mS]; % the matrices for:
              % (1,2) | B(w) - exp(-tau*w) A(w) | <= gp * |A(w)|, w in [0, wp]
              % (3,4) | B(w) | <= gs * |A(w)|, w in [ws, pi]

nrA = 2*N1+1 + 2*m1+1; % two LMIs for each norm inequality (BRL)
ncA = 1 + m1 + NQ2 + NS2 + mQ2 + mS2;
if gs > 0
  nrA = nrA + 1;
end

As = sparse(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(1) = 1;    % minimize stopband error

% Ra(z) = A(z)*A(z^{-1)), only distinct coefficients
a = a(:);
Ra = zeros(N1,1);     % leave space for maximal degree
for k = 1 : n1
  Ra(k) = a(1:n1+1-k)' * a(k:end);
end

% BRL 1: | B(w) - exp(-tau*w) A(w) | <= gp * |A(w)|, w in [0, wp]
sc = 1+m1;      % start column in As for this BRL matrices
rr = 1:N1;      % row range in As for the first LMI
[A1,B1] = posintlmi_r(N, 0, wp);
for k = 1 : N1    % resize the first coefficient matrices
  A2 = sparse(NQ,NQ);
  A2(1:N1,1:N1) = mat( A1(k,:)' );
  As(k, sc+1:sc+NQ2) = vec(A2)';
end
As(rr, sc+NQ2+1:sc+NQ2+NS2) = B1;
bs(rr) = gp*gp * Ra;

% second LMI of BRL 1
%sr = N1+1;
for k = 1 : N1    % the last row of the matrix is b - a shift tau
  A2 = sparse(NQ,NQ);
  A2(NQ,k) = 1;
  As(N1+k, sc+1:sc+NQ2) = vec(A2)';
  As(N1+k, k+1) = -1;  % coefficient of b
  %sr = sr + 1;
end

% contribution of a
for k = 1 : n1
  bs(N1+k+tau) = -a(k);  % the group delay contribution
end

% the lower right element is 1
A2 = sparse(NQ,NQ);
A2(NQ,NQ) = 1;
As(2*N1+1, sc+1:sc+NQ2) = vec(A2)';
bs(2*N1+1) = 1;

% BRL 2: | B(w) | <= gs * |A(w)|, w in [ws, pi]
sc = 1+m1+NQ2+NS2;    % start column in As for this BRL matrices
rr = 2*N1+2 : 2*N1+1+m1; % row range in As for the first LMI
[A1,B1] = posintlmi_r(m, ws, 1);
for k = 1 : m1    % resize the first coefficient matrices
  A2 = sparse(mQ,mQ);
  A2(1:m1,1:m1) = mat( A1(k,:)' );
  As(2*N1+1+k, sc+1:sc+mQ2) = vec(A2)';
end
As(rr, sc+mQ2+1:sc+mQ2+mS2) = B1;
As(rr,1) = -Ra(1:m1);      % gs*gs*Ra is lht of the LMI !

% second LMI of BRL 2
sr = 2*N1+1+m1;
for k = 1 : m1    % the last row of the matrix is b
  A2 = sparse(mQ,mQ);
  A2(mQ,k) = 1;
  As(sr+k, sc+1:sc+mQ2) = vec(A2)';
  As(sr+k, k+1) = -1;
end

% the lower right element is 1
A2 = sparse(mQ,mQ);
A2(mQ,mQ) = 1;
As(sr+m1+1, sc+1:sc+mQ2) = vec(A2)';
bs(sr+m1+1) = 1;

% keep gs fixed, if feasibility problem
if gs > 0
  As(end,1) = 1;
  bs(end) = gs*gs;
end

%spy(As)

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

gs = sqrt(x(1));
b = x(2:m1+1);  % the numerator
