function [a, b, cost, count] = newton_prstab(n, m, rho, freq, dH, weight, a, b, max_count)

% Descent method optimization IIR filter with least squares criterion
% Input:
%   n		        - denominator degree
%   m	      	  - numerator degree
%   rho		      - radius of circle where the roots must hold
%   freq	      - frequency grid, L points
%   dH		      - desired frequency response, on the grid
%   weight	    - the weight associated with the frequency points
%   a0,b0       - initialization (optional)
%   max_count   - maximum number of iterations (implicit 100)
% Output:
%   a,b         - optimized denominator and numerator
%   cost        - criterion value
%   iter        - number of iterations actually performed

% Riitta Niemisto 28.1.2002
% BD 28.07.2006 (revised)

disp('Refinement:')

tol = 1e-5;
err = 2*tol;
t = cputime;

if nargin < 9
  max_count=100;
end
if nargin < 7
  a = zeros(1,n);
  b = eye(1,m1);
else
  b = b(:)' / a(1);
  a = a(:)' / a(1);
  a = a(2:end);       % ignore first coefficient (=1)
end

la1 = 1:n;
lb = 0:m;
N = length(dH);

cost = J2(weight, dH, freq, b, [1 a]);
cost_old = 2 * cost;
grad = zeros(n+m+1,1);

count=0;
last='Hess';

disp([count cost])

while  ((cost_old-cost)/cost_old > tol) && (count < max_count)

  grad_old=grad;
  [grad, H] = Jgradhes(b, a, freq, dH, weight);
  if rcond(H) > 1e-10;
    delta = (-H\grad)';
    
    delta_b = delta(1:m+1);
    delta_a = delta(m+2:end);
  
    alpha_in = 0.99 * max_alpha(delta_a,a,rho);
    if alpha_in >0
      alpha = min_gold(weight,dH,freq,b,a,delta_b,delta_a,alpha_in);
    else
      alpha=0;
    end
  else
    alpha=0;
  end
  
  if alpha == 0			% gradient direction
    if last=='Hess'
      disp('Gradient')
      delta = -grad';
      last='grad';
    else 
      if last=='grad'
	disp('Conjugate Gradient')
      end
      gamma = (grad-grad_old)'*grad/(grad_old'*grad_old);
      delta = -grad'+gamma*delta;
      last='conj';
    end
    delta_b = delta(1:m+1);
    delta_a = delta(m+2:end);
  
    alpha_in = 0.99 * max_alpha(delta_a,a,rho);
    if alpha_in >0
      alpha = min_gold(weight,dH,freq,b,a,delta_b,delta_a,alpha_in);
    else
      alpha=0;
    end
   else
    if last~='Hess'
      last='Hess';
      disp('Hess')
    end
  end
  a_new = a+alpha*delta_a;

  max_pole=max(abs(roots([1 a_new])));
  
  if max_pole > rho
    disp('Roots outside the circle !!!')
    disp(max_pole)
    break;
  end

  a = a_new;
  b = iir_ls_fixa( m, [1 a], freq, dH, weight)';

  count = count+1;
  cost_old = cost;
  cost = J2(weight, dH, freq, b, [1 a]);
  err = (cost_old-cost)/cost_old;
  
  disp([count cost err alpha/alpha_in*100 cputime-t]) 
end

%b=b';
a=[1 a];
