function [alpha] = max_alpha_2d(delta, a, rho, inda)
%
%   alpha = max_alpha(delta,a,rho)
%
%  Computes the maximal alpha for which
%
%      alpha * delta (rho_1 z_1, rho_2 z_2)
%  1 + ------------------------------------- >= 0
%           a (rho_1 z_1, rho_2 z_2)
%
% Arguments:
% a		- a stable 2D polynomial
% delta		- a 2D polynomial
% rho		- [rho_1, rho_2], robustness margin
% inda		- what type of 2D filter (quarter plane=0, half plane>0)
%
% Note: a and delta are completed with values according to inda,
% e.g. inda=0 implies a(1,1)=1, delta(1,1)=0.

%  BD 19.02.2002

[n1,n2] = size(a);
[m1,m2] = size(a);

if m1 ~= n1 | m2 ~= n2
  error('a and delta must have same size');
end

if nargin < 3
  rho = [1 1];
end
if nargin < 4
  inda = 0;		% quarter plane
end

a(1,1) = 1;
delta(1,1) = 0;

m1 = 2*n1 - 1;
r1 = (n2-1)*m1 + n1;	% index of first useful row in convolution matrix

rhov1 = rho*ones(n1,1);
rhov1(1) = 1;
rhov1 = cumprod(rhov1);
rhov2 = rho*ones(n2,1);
rhov2(1) = 1;
rhov2 = rhov1(end) * cumprod(rhov2);
R = hankel(rhov1,rhov2);
a = a ./ R;             % !!!!!!!!!
delta = delta ./ R;     % !!!!!!!!!

% build matrices for equality constraints
aa = reshape( fliplr(a(:)')', n1, n2 );
F = convmtx2( aa, n1, n2 );	% block toeplitz, with reversed columns
F = F( r1:end, :);		% keep only significant rows

G = convmtx2( a, n1, n2 );
G = G( r1:end, : );
G = G(:,end:-1:1);		% block Hankel with Hankel blocks !

f = 2 * G * a(:);

u = -(F+G)*delta(:);

N = n1 * n2;
M = N + (n1-1)*(n2-1);

% build 2D Toeplitz matrices
A = zeros(M, 1+N^2);
A(:,1) = u;

k1=0 ; k2=0;
k = 1;
while k1 < n1 & k2 < n2
  A(k, 2:end) = vec(kron(diag(ones(n2-k2,1),-k2), diag(ones(n1-abs(k1),1),-k1)))';
  k1 = k1 + 1;
  if k1 == n1
    k1 = -n1+1;
    k2 = k2+1;
  end
  k = k+1;
end

b = f;
c = [-1;
     zeros(N^2,1)];

K.l=1;
K.s = [N];

pars.fid = 0;
pars.eps = 1e-10;

[x,y,info] = sedumi(A,b,c,K,pars);
%info

alpha=x(1);

