function [mu,Q1,Q2] = minpol1_g2(r)

% Compute the minimum value on the unit circle of a real trigonometric polynomial
% Method: 2-Gram matrix parameterization
% Input:
%   R       - vector of distinct coefficients, starting with free term
% Output:
%   mu      - the minimum
%   Q1,Q2   - most positive Gram pair matrices associated with the polynomial

% Return:
% mu        - the minimum

% BD 17.11.2005

%t0 = cputime;

if ~isreal(r)       % complex data
  error('Only real polynomials with this method')
end

n = length(r);      % length of the polynomial
odd_order = 0;
if rem(n,2) == 0    % odd order
  odd_order = 1;
end
m = floor((n-1)/2); % half the order
m1 = m+1;

Ks.f = 1;           % one free variables (mu)
nrA = n;            % number of equality constraints (one for each coefficient of r)
if odd_order
  Ks.s = [m1 m1];   % two positive semidefinite matrices (Q1 and Q2)
  ncA = 1+2*m1*m1;  % number of (scalar) variables (in mu, Q1, Q2)
else
  Ks.s = [m1 m];    % two positive semidefinite matrices (Q1 and Q2)
  ncA = 1+m1*m1+m*m;% number of (scalar) variables (in mu, Q1, Q2)
end

As = sparse(nrA,ncA);
bs = r(:);          % the right hand term of the equality constraints
cs = zeros(ncA,1);
cs(1) = -1;         % the objective is to maximize mu

warning off MATLAB:hankel:AntiDiagonalConflict
nc1 = 2 : 1+m1*m1;  % column range for Q1 and Q2
nc2 = 2+m1*m1 : ncA;
e = ones(m1,1);

% first row - for r0
if odd_order
  u = 0.5 * vec( spdiags(e,0,m1,m1) );
  As(1,nc1) = u';
  As(1,nc2) = u';
else
  As(1,nc1) = 0.5 * vec( spdiags(e,0,m1,m1) )'; % the "Toeplitz" part
  As(1,2) = 1;                                  % plus the "Hankel" part
  As(1,nc2) = 0.5 * vec( spdiags(e,0,m,m) )';
end

% the other rows of As
if odd_order
  for k = 2:n
    v = zeros(n-1,1);
    v(k-1) = 1;
    u = 0.25 * vec( sparse(hankel(v(1:m1), v(m1:end))) );
    As(k,nc1) = u';
    As(k,nc2) = -u';
    if k <= m1
      u = 0.5 * vec( spdiags(e,k-1,m1,m1) );
      As(k,nc1) = As(k,nc1) + u';
      As(k,nc2) = As(k,nc2) + u';
    end
  end
else              % even order
  for k = 2:n
    v = zeros(n,1);
    v(k) = 1;
    As(k,nc1) = 0.25 * vec( sparse(hankel(v(1:m1), v(m1:end))) )';
    if k <= m1
      As(k,nc1) = As(k,nc1) + 0.5 * vec( spdiags(e,k-1,m1,m1) )';
    end
    if k > 2
      As(k,nc2) = - 0.25 * vec( sparse(hankel(v(3:m+2), v(m+2:end))) )';
    end
    if k <= m
      As(k,nc2) = As(k,nc2) + 0.5 * vec( spdiags(e,k-1,m,m) )';
    end
  end
end
As(1,1) = 1;

%prep_time = cputime - t0

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

mu = x(1);        % recover the variables
Q1 = mat(x(nc1));
Q2 = mat(x(nc2));
