function mu = minpol2_dom(R, nD, D, nplus)

% Compute the minimum value of a 2D trigonometric polynomial on a union of frequency
% domains defined by the common positivity of some trigonometric polynomials
% Input:
%   R       - polynomial coefficients, symmetric matrix (all coefficients)
%   nD      - number of polynomials for each domain in the union;
%             if e.g. nD = [3 4], then length(D) = 7 and the admissible domain 
%             is the union of two domains, one defined by 3 polynomials (the
%             first 3 from D) and the other defined by 4 polynomials
%   D       - cell vector, each cell containing the coefficients of a
%             polynomial defining the domain (symmetric matrix)
%   nplus   - extra degree for relaxation (implicit value is zero)
% Output:
%   mu      - minimum value of the polynomial, on the given domain

% BD 12.10.2005

%---------------------------------------------------------
if nargin < 4
  nplus = [0 0];
end

% sizes, orders and numbers...
Rsize = (size(R) + 1) / 2;
nh = Rsize + nplus;
Rbig = zeros(size(R) + 2*nplus);
Rbig(1+nplus(1):end-nplus(1),1+nplus(2):end-nplus(2)) = R;
R = Rbig;

nlmi = length(nD);     % number of LMIs
nPmat = sum(nD);       % number of P matrices (for each LMI)

nd = zeros(2,nPmat);  % orders of D polynomials
for i = 1 : nPmat
  nd(1,i) = (size(D{i},1)-1) / 2;
  nd(2,i) = (size(D{i},2)-1) / 2;
end

np = repmat([nh(1); nh(2)],1,nPmat)  - nd;    % sizes (!) of S polynomials

Np = np(1,:) .* np(2,:);
Nps = Np.*Np;

N = nh(1) * nh(2);            % number of polynomial coefficients
M = N + (nh(1)-1)*(nh(2)-1);  % number of "trace" constraints
                              % (and of distinct filter coefficients)

Nhs = N*N;

vcoef = vec(R);
vcoef = vcoef(1:M);         % keep only the distinct coefs
vcoef = vcoef(end:-1:1);    % put them in the correct order

% here starts the SDP problem description
nrA = M*nlmi;               % number of equality constraints
                            % (M for each LMI, plus Mz for zero coefs)
ncA = 1+ nlmi*Nhs + sum(Nps);     % number of variables

Ks.f = 1;                   % minimum value of polynomial
Ks.s = [];
icAlmi = zeros(1,2*nlmi);
k=0;
for i = 1 : nlmi            % for each LMI
  icAlmi(i) = length(Ks.s)+1;
  v = [];
  v = [v N];                % the size of Q0 matrix
  v = [v Np(k+1:k+nD(i))];  % and of the other Q matrices
  Ks.s = [Ks.s v];
  k = k+nD(i);
end

As = spalloc( nrA, ncA, ceil(nrA*ncA/20) );  % allocate some space...
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );
cs(1) = -1;                 % maximization (of free variable)

irAs = 0 : M : nlmi*M;
icAs = cumsum([0 Ks.s.*Ks.s]) + 1;

%-------------------------------------
% try speeding-up by storing Toeplitz matrices
for i = 1 : 2*nh(1)-1
  Ts1{i} = spdiags( ones(nh(1),1), i-nh(1), nh(1), nh(1) );
end
for i = 1 : 2*nh(2)-1
  Ts2{i} = spdiags( ones(nh(2),1), i-nh(2), nh(2), nh(2) );
end
%-------------------------------------

% the "trace" (Q) part of the positivity equalities
k1=0 ; k2=0;
i = 1;
while k1 < nh(1) & k2 < nh(2)
  Z = spalloc(N,N,N);
  Z = kron(Ts2{k2+nh(2)},Ts1{k1+nh(1)});
  v = vec(Z)';
  for ii = 1 : nlmi
    As((ii-1)*M+i, icAs(icAlmi(ii))+1:icAs(icAlmi(ii)+1)) = v;
  end
  k1 = k1 + 1;
  if k1 == nh(1)
    k1 = -nh(1)+1;
    k2 = k2+1;
  end
  i = i+1;
end

% the free term (the coefficients of the polynomial)
bs = repmat(vcoef, nlmi, 1);

% the free variable (the minimum value)
for ii = 1 : nlmi
  As((ii-1)*M+1, 1) = 1;
end

% the "Qell" part of the LMIs
id = 0;
warning off MATLAB:toeplitz:DiagonalConflict
for il = 1 : nlmi       % for each LMI
  for ip = 1 : nD(il)   % for each P matrix in the LMI
    id = id + 1;
    % build the convolution matrices for D polynomials
    clear Td
    tr = zeros(2*np(1,id)-1,1);
    for k2 = 1 : size(D{id},2)
      tc = zeros(2*nh(1)-1,1);
      tc(1:size(D{id},1)) = D{id}(:,k2);
      Z = toeplitz(tc,tr);
      Td{k2} = sparse(Z);
    end
    
    % the trace matrices for the "Qell" positive polynomials
    for k2 = 1 : nh(2)
      Tp{k2} = spalloc(2*np(1,id)-1, Nps(id), Nps(id));
      for p = 1 : 2*np(1,id)-1
        Z = kron(Ts2{nh(2)-nd(2,id)+k2-1}(1:np(2,id), 1:np(2,id)), ...
                 Ts1{nh(1)-np(1,id)+p}(1:np(1,id), 1:np(1,id)));
        Tp{k2}(p,:) = vec(Z)';
      end
    end

    % store in As the "Qell" part of the LMIs
    % the first "half" row (of the half plane)
    z = sparse(2*nh(1)-1,Nps(id));
    nd22 = size(D{id},2);
    for ii = 1 : nd22
      if nd22+1-ii <= nh(2)
        z = z + Td{ii} * Tp{nd22+1-ii};
      end
    end
    As((il-1)*M+1:(il-1)*M+nh(1), icAs(icAlmi(il)+ip)+1:icAs(icAlmi(il)+ip+1)) = z(nh(1):end,:);

    % the other coefficients
    is = nh(1);
    for k2 = 1 : nh(2)-1
      z = sparse(2*nh(1)-1,Nps(id));
      for ii = max(1, nd22+1+k2-nh(2)) : nd22
        z = z + Td{ii} * Tp{nd22+1+k2-ii};
      end
      As((il-1)*M+is+1:(il-1)*M+is + 2*nh(1)-1, icAs(icAlmi(il)+ip)+1:icAs(icAlmi(il)+ip+1)) = z;
      is = is + 2*nh(1)-1;
    end

  end
end

% call Sedumi
%pars.fid = 0;
pars.eps = 1e-8;

[x,y,info] = sedumi(As,bs,cs,Ks,pars);
info

mu = x(1);
