function [mu,Q1,Q2] = minpol2_g2(polcoef)

% Compute the minimum value on the unit circle of a 2-D real trigonometric polynomial
% Method: 2-Gram matrix parameterization
% Input:
%   R       - polynomial coefficients, symmetric matrix (all coefficients) 
% Output:
%   mu      - the minimum
%   Q1,Q2   - most positive Gram pair matrices associated with the polynomial

% BD 21.11.2005

t0 = cputime;

if ~isreal(polcoef)         % complex data
  error('Only real polynomials with this method')
end

Rdeg = (size(polcoef)-1) / 2; % degree of the polynomial

% parity_deg = 0    ==>  Rdeg(2) even, Rdeg(1) even
%              1         Rdeg(2) odd,  Rdeg(1) even
%              2         Rdeg(2) odd,  Rdeg(1) odd
parity_deg = rem(Rdeg(1),2) + rem(Rdeg(2),2)
if parity_deg == 1 & rem(Rdeg(1),2) == 1
  polcoef = polcoef';       % swap variables (to have only 3 parity cases)
  Rdeg = (size(polcoef)-1) / 2; % recompute
end
nd2 = size( polcoef );
Rsize = Rdeg + 1;           % size of polynomial

m = floor(Rdeg/2);          % half the order
m1 = m+1;
N = (prod(nd2)+1) / 2;      % number of coefs in symmetric polynomial

switch parity_deg
  case 0,
    M = (prod(2*m+1)+1) / 2;    % number of coefs for a sos factor
    M2 = M*M;
    M1 = M-1;
    M12 = M1*M1;

    Ks.s = [M M1];  % two positive semidefinite matrices (Q1 and Q2)
    ncA = 1+M2+M12; % number of (scalar) variables (in mu, Q1, Q2)

  case 1,
    M = (2*m(1)+1) * (m(2)+1);
    M2 = M*M;

    Ks.s = [M M];   % two positive semidefinite matrices (Q1 and Q2)
    ncA = 1+M2+M2;  % number of (scalar) variables (in mu, Q1, Q2)
    
  case 2,
    M = 2 * prod(m1);
    M2 = M*M;

    Ks.s = [M M];   % two positive semidefinite matrices (Q1 and Q2)
    ncA = 1+M2+M2;  % number of (scalar) variables (in mu, Q1, Q2)
end

r12 = 2*Rdeg(1) + 1;
   
nc1 = 2 : 1+M2;  % column range for Q1 and Q2
nc2 = 2+M2 : ncA;
c2s = 1+M2;

Ks.f = 1;           % one free variables (mu)
nrA = N;            % number of equality constraints (one for each coefficient of r)

As = sparse(nrA,ncA);

vcoef = vec(polcoef);
vcoef = vcoef(1:N);         % keep only the distinct coefs
vcoef = vcoef(end:-1:1);    % put them in the correct order
bs = vcoef;

cs = zeros(ncA,1);
cs(1) = -1;         % the objective is to maximize mu
As(1,1) = 1;        % the coefficient of mu

switch parity_deg
  case 0,
    iv = (1:M)';        % vector of indices in half of halfplane
    ii = m(1)+1;
    kk = r12 - m(1);
    for i2 = 1 : m(2)
      iv( ii+1 : ii+Rsize(1) ) = kk+1 : kk+Rsize(1);
      ii = ii + Rsize(1);
      kk = kk + r12;
    end
    %iv'
    
    dh1 = -1;
    iv2 = iv(2:end);        % no free term for sines

  case 1,
    iv = zeros(M,1);        % vector of indices in half of halfplane
    ii = 0;
    kk = -m(1);
    for i2 = 0 : m(2)
      iv( ii+1 : ii+Rsize(1) ) = kk+1 : kk+Rsize(1);
      ii = ii + Rsize(1);
      kk = kk + r12;
    end
    %iv'
    
    dh1 = r12-1;
    iv2 = iv;               % same number of terms in sine base
    
  case 2,
    iv = zeros(M,1);        % vector of indices in half of halfplane
    ii = 0;
    kk = -m(1)-1;
    for i2 = 0 : m(2)
      iv( ii+1 : ii+Rsize(1) ) = kk+1 : kk+Rsize(1);
      ii = ii + Rsize(1);
      kk = kk + r12;
    end
    %iv'
    
    dh1 = r12;
    iv2 = iv;               % same number of terms in sine base
end

% the Q1 part
for i = 1 : M
  for j = 1 : M
    % the Hankel part
    k = iv(i) + iv(j) + dh1;    % the affected coefficient
    if k == 1, ad = 0.5; else, ad = 0.25; end
    d = M*(j-1) + i;
    As(k, 1 + d ) = As(k, 1 + d ) + ad;
    % the Toeplitz part
    k = abs(iv(i) - iv(j)) + 1;
    if k == 1, ad = 0.5; else, ad = 0.25; end
    As(k, 1 + d ) = As(k, 1 + d ) + ad;
  end
end

% the Q2 part
Liv2 = length(iv2);
for i = 1 : Liv2
  for j = 1 : Liv2
    % the Hankel part
    k = iv2(i) + iv2(j) + dh1;    % the affected coefficient
    if k == 1, ad = 0.5; else, ad = 0.25; end
    d = Liv2*(j-1) + i;
    %c2s+d
    As(k, c2s + d ) = As(k, c2s + d ) - ad;
    % the Toeplitz part
    k = abs(iv2(i) - iv2(j)) + 1;
    if k == 1, ad = 0.5; else, ad = 0.25; end
    As(k, c2s + d ) = As(k, c2s + d ) + ad;
  end
end

%full(mat(As(3, nc1)))
%full(mat(As(3, nc2)))
%sum(sum(As(:,nc1)))
%spy(As)
%mesh(As)

%mu=0; return

prep_time = cputime - t0

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

mu = x(1);        % recover the variables
Q1 = mat(x(nc1));
Q2 = mat(x(nc2));
