function [mu,Q,S] = minpol2_g2_even(R)

% [mu,Q,S] = minpol2_g2(R)
%
% Compute the minimum value on the unit circle of a 2-D real trigonometric polynomial
% Method: 2-Gram matrix parameterization
% Warning: works only for even degree !!!
% Input:
%   R       - polynomial coefficients, symmetric matrix (all coefficients)
% Output:
%   mu      - the minimum
%   Q,S     - most positive Gram pair matrices associated with the polynomial

% BD 13.12.2005

ns = size(R);           % storage size of the polynomial
n = (ns-1) / 2;         % degree of the polynomial
if mod(n(1),2) == 1 || mod(n(2),1) == 1
  error('This program works only for even degree')
end
nh = floor(n/2);        % half the order
M = (prod(ns)+1)/2;     % number of coefs in symmetric polynomial
Nc = (prod(n+1)+1)/2;   % number of coefs for cos base
Ns = Nc - 1;            % number of coefs for sin base

K.f = 1;                % one free variables (mu)
K.s = [Nc Ns];          % two positive semidefinite matrices (Q and S)
nrA = M;                % number of equality constraints (one for each coefficient of r)
ncA = 1+Nc*Nc+Ns*Ns;    % number of (scalar) variables (in mu, Q, S)

c2s = 1+Nc*Nc;          % end column of Q, in A
ncQ = 2 : c2s;          % column range for Q, in A
ncS = 1+c2s : ncA;      % column range for S, in A

A = sparse(nrA,ncA);

vR = vec(R);
vR = vR(1:M);           % keep only the distinct coefs
vR = vR(end:-1:1);      % put them in the correct order
b  = vR;                % the r.h.s. of the equality constraints

c = zeros(ncA,1);
c(1) = -1;              % the objective is to maximize mu
A(1,1) = 1;             % the coefficient of mu

iv = (1:Nc)';           % vector of indices for support of \tilde H(z)
ii = nh(1)+1;
kk = 2*n(1) + 1 - nh(1);
for i2 = 1 : nh(2)
  iv( ii+1 : ii+n(1)+1 ) = kk+1 : kk+n(1)+1;
  ii = ii + n(1)+1;
  kk = kk + 2*n(1) + 1;
end
    
% the Q part
for ii = 1 : Nc
  for ll = 1 : Nc
    k = iv(ii) + iv(ll) - 1;      % coef affected by sum
    if k == 1, ad = 0.5; else, ad = 0.25; end
    d = Nc*(ll-1) + ii;
    A(k, 1 + d ) = A(k, 1 + d ) + ad;
    k = abs(iv(ii) - iv(ll)) + 1; % coef affected by difference
    if k == 1, ad = 0.5; else, ad = 0.25; end
    A(k, 1 + d ) = A(k, 1 + d ) + ad;
  end
end

% the S part
iv = iv(2:end);      % no free term for sines
for ii = 1 : Ns
  for ll = 1 : Ns
    k = iv(ii) + iv(ll) - 1;      % coef affected by sum
    if k == 1, ad = 0.5; else, ad = 0.25; end
    d = Ns*(ll-1) + ii;
    A(k, c2s + d ) = A(k, c2s + d ) - ad;
    k = abs(iv(ii) - iv(ll)) + 1; % coef affected by difference
    if k == 1, ad = 0.5; else, ad = 0.25; end
    A(k, c2s + d ) = A(k, c2s + d ) + ad;
  end
end

%full(mat(A(12, ncQ)))
%full(mat(A(12, ncS)))
%sum(sum(A(:,nc1)))
%spy(A)

% call SeDuMi
pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(A,b,c,K,pars); % call SeDuMi
info              % display run statistics

mu = x(1);        % recover the variables
Q = mat(x(ncQ));
S = mat(x(ncS));
