function mu = minpol_cvx(R, nR, K, nD, D)

% Compute the minimum value on the unit circle of a multivariate trigonometric
% polynomial, using a sum-of-square relaxation.
% If the polynomial has matrix coefficients, then the minimum value of the
% smallest eigenvalue is computed.
% The optimization can be carried on a union of domains, each defined by the positivity of
% some trigonometric polynomials
%
% Input:
%   R       - vector of distinct coefficients (causal part, starting with
%             free coefficient)
%   nR      - degree of the polynomial (not necessary in the univariate case)
%   K       - size of matrix coefficients is KxK (default value is 1)
%   nD      - number of polynomials for each domain in the union.
%             (If missing, the minimum is computed on the whole unit circle.)
%             if e.g. nD = [3 4], then length(D) = 7 and the admissible domain 
%             is the union of two domains, one defined by 3 polynomials (the
%             first 3 from D) and the other defined by 4 polynomials
%   D       - a description of the domain.
%             Univariate polynomials: D is an interval [a,b]
%               represented by a vector with 2 elements.
%             Multivariate: D is a cell vector, each element being a structure.
%               D{i}.p - symmetric polynomial represented in vector form (see pos_trig_poly).
%               D{i}.d - is the degree of the polynomial
% Output:
%   mu      - the minimum

% BD 2.05.2010

unconstrained_opt = 0;
if nargin < 4
  unconstrained_opt = 1;
end
if nargin < 3
  K = 1;
end
if nargin < 2
  nR = length(R)-1;
end

n = length(R);
R = R(:);
Ivec = zeros(n,1);
K2 = K*(K+1)/2;
Ivec(1:K2) = vecs(eye(K));
real_coef = 1;
if ~isreal(R)
  real_coef = 0;
end
cvx_begin
  variable mu;        % the minimum value of the polynomial
  if unconstrained_opt
    if real_coef
      variable Rp(n);
      Rp == sos_trig_poly(nR,K);
    else
      variable R(n) complex;
      Rp == sos_trig_poly(nR,K,'c');
    end
    maximize mu;
    subject to
      mu*Ivec + Rp == R;
  else  % constrained optimization
    ndom = length(nD);  % number of domains in the union
    variable mu_vec(ndom);  % vector of slack variables
    for i = 1 : ndom
      if real_coef
        eval(['variable Rp' num2str(i) '(n)']);
        eval(['Rp' num2str(i) '== pos_trig_poly(nR,K,D)']);
      else
        eval(['variable Rp' num2str(i) '(n) complex']);
        eval(['Rp' num2str(i) '== pos_trig_poly(nR,K,D,''c'')']);
      end      
    end
    maximize mu;
    subject to
      mu <= mu_vec;
      for i = 1 : ndom
        eval( ['mu_vec(i)*Ivec + Rp' num2str(i) ' == R'] );
      end
  end  
cvx_end
