function [Q, mineig, minpol, gap] = most_pos_Gram(polcoef, nd)

% Find the Gram matrix associated with a multivariate trigonometric polynomial
% whose minimum eigenvalue is maximum
% Input:
%   polcoef - polynomial coefficients; if this is the single argument, it is assumed
%             that the coefficients are fully stored, even though the polynomial is symmetric;
%             if two arguments, then the coefficients belong to a half space and are vectorized
%   nd      - degree of the polynomial
% Output:
%   Q      - most positive Gram matrix
%   mineig - minimum eigenvalue of the Gram matrix
%   minpol - minimum value of the polynomial on the unit circle
%   gap    - duality gap for computed solution

% BD 12.09.2005

% sizes and numbers
if nargin == 1
  nd2 = size( polcoef );      % 2*nd + 1 (storage convention)
  nvar = length( nd2 );       % number of variables
  Rsize = (nd2+1) / 2;        % nd + 1
  M = prod(nd2);
  N = (M+1)/2;                % number of distinct coefs in symmetric polynomial
  polcoef = vec(polcoef);
  polcoef = polcoef(1:N);         % keep only the distinct coefs
  polcoef = polcoef(end:-1:1);    % put them in the correct order
else
  polcoef = polcoef(:);
  nd2 = 2*nd + 1;
  nvar = length(nd);
  Rsize = nd + 1;
  N = length(polcoef);
  M = prod(nd2);
  if N ~= (M+1)/2
    error('Disagreement between length of coefficients vector and degree')
  end
end
nG = prod(Rsize);           % size of Gram matrix

% Sedumi parameters
nrA = N;                    % number of equality constraints
ncA = 1 + nG*nG;            % number of variables

Ks.f = 1;                   % the eigenvalue (real number)
Ks.s = nG;

As = spalloc( nrA, ncA, 2*nG*nG );  % very approximate allocation
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );

bs = polcoef;               % !!!
cs(1) = -1;                 % maximize the eigenvalue

%-------------------------------------
% try speeding-up by storing Toeplitz matrices
dmax = max(Rsize);
for i = 1 : 2*dmax-1
  Ts{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
end
%-------------------------------------
% generalized "trace" equalities
ki = zeros(nvar,1);
k = 1;
for kk = 1 : N
  T = Ts{ki(1)+dmax}(1:Rsize(1),1:Rsize(1));
  for i = 2 : nvar
    T = kron( Ts{ki(i)+dmax}(1:Rsize(i),1:Rsize(i)), T );
  end
  T = (T+T')/2;
  As(k,1) = trace(T);       % eigenvalue coefficient
  T = vec(T);
  As(k, 2:end) = T';        % "trace" matrix
  k = k+1;
  ki(1) = ki(1) + 1;        % go to next (poly)index
  for i = 1 : nvar - 1
    if ki(i) == Rsize(i)
      ki(i) = -Rsize(i)+1;
      ki(i+1) = ki(i+1) + 1;
    else
      break;
    end
  end
end

%======================================================

% call Sedumi
pars.fid = 0;
pars.eps = 1e-10;

[x,y,info] = sedumi(As,bs,cs,Ks,pars);

mineig = x(1);
Q = mat(x(2:end)) + mineig*eye(nG);
info
gap = abs(cs'*x - bs'*y);
minpol = mineig * nG;

