function [Q, mineig, minpol, gap] = most_pos_Gram_M(polcoef, nd)

% Find the Gram matrix associated with a multivariate symmetric polynomial
% with matrix coefficients, whose minimum eigenvalue is maximum
% Input:
%   polcoef - polynomial coefficients, fully stored, even though the polynomial is symmetric;
%             polcoef is an array of cells, each matrix coefficient is a cell
%   nd      - degree of the polynomial
% Output:
%   Q      - most positive Gram matrix
%   mineig - minimum eigenvalue of the Gram matrix
%   minpol - minimum value of the polynomial on the unit circle
%   gap    - duality gap for computed solution

% BD 3.10.2005
% BD 27.02.2009 (complex case added)

% sizes and numbers
nvar = length( nd );     % number of variables
Rsize = nd+1;
nd2 = 2*Rsize - 1;
nb = size(polcoef{1,1},1);

% start multidimensional test
nG = nb * prod(Rsize);      % size of Gram matrix
M = prod(nd2);
N = (M+1)/2;                % number of coefs in symmetric polynomial
%vcoef = vec(polcoef);
%vcoef = vcoef(1:N);         % keep only the distinct coefs
%vcoef = vcoef(end:-1:1);    % put them in the correct order

% Sedumi parameters
nrA = N*nb*nb - nb*(nb-1)/2;   % number of equality constraints
ncA = 1 + nG*nG;            % number of variables

Ks.f = 1;                   % the eigenvalue (real number)
Ks.s = nG;

if ~isreal(polcoef)   % if complex coefficients
  Ks.scomplex = 1;    % the positive definite matrix is complex
  Ks.ycomplex = 1:nrA;% and all equalties are complex
end

As = spalloc( nrA, ncA, 2*nG*nG );  % very approximate allocation
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );

%bs = vcoef;                 % !!!
cs(1) = -1;                 % maximize the eigenvalue

%-------------------------------------
% store block selectors
for i = 1 : nb
  for j = 1 : nb
    blS{i,j} = zeros(nb);
    blS{i,j}(i,j) = 1;
%    blS{i,j}
  end
end
%-------------------------------------
% try speeding-up by storing Toeplitz matrices
dmax = max(Rsize);
for i = 1 : 2*dmax-1
  Ts{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
end
%-------------------------------------
% generalized "trace" equalities
ki = zeros(nvar,1);
kb = 1;
k = 1;
yreal = [];
for kk = 1 : N
  T = Ts{ki(1)+dmax}(1:Rsize(1),1:Rsize(1));
  for i = 2 : nvar
    T = kron( Ts{ki(i)+dmax}(1:Rsize(i),1:Rsize(i)), T );
  end
%  T = (T+T')/2;
  for i = 1 : nb            % for the elements of a block
    for j = 1 : nb
      if kb > 1 || i >= j
        TT = kron(T,blS{i,j});
        %TT = (TT+TT')/2;
        As(k,1) = trace(TT);       % eigenvalue coefficient
        TT = vec(TT);
        As(k, 2:end) = TT';        % "trace" matrix
        bs(k) = polcoef{N-kb+1}(i,j);
        if kb == 1 && i == j
          yreal = [yreal k];
        end
        k = k+1;
      end
    end
  end

  kb = kb + 1;
  ki(1) = ki(1) + 1;        % go to next (poly)index
  for i = 1 : nvar - 1
    if ki(i) == Rsize(i)
      ki(i) = -Rsize(i)+1;
      ki(i+1) = ki(i+1) + 1;
    else
      break;
    end
  end
end
Ks.ycomplex(yreal) = []; % eq. involving the diagonal are real
%spy(As)
%bs'

%======================================================

% call Sedumi
pars.fid = 0;
pars.eps = 1e-10;

[x,y,info] = sedumi(As,bs,cs,Ks,pars);

mineig = x(1);
Q = mat(x(2:end)) + mineig*eye(nG);
info
gap = abs(cs'*x - real(bs'*y));
minpol = mineig * size(Q,1) / nb;

