function [ninf,Q] = norm_fir_mimo_brl(H)

% Compute the infinity norm of a FIR MIMO discrete-time
% linear system.
% Method: efficient form of the BRL lemma.
% Input:
%   H     - array containing the matrix coefficients of the
%           FIR MIMO system, in increasing order of powers
% Output:
% ninf    - the infinity norm of the system

% BD 23.11.2004

N = length(H);      % length of FIR system
[m,p] = size(H{1});

if m < p      % transpose for smaller size
  for n = 1 : N
    H{n} = H{n}';
  end
  [m,p] = size(H{1});
end

Ng = N*p + m;         % size of Gram matrix

Ks.l = 1;
Ks.s = [Ng];
nrA = N*m*p + (N-1)*p*p + m*(m+1)/2 + p*(p+1)/2;
ncA = 1 + Ng*Ng;

As = sparse(nrA,ncA);
bs = zeros(nrA,1);
cs = zeros(ncA,1);
cs(1) = 1;

% the equalities for the coefficients of H
ia = 1;             % row index in As
for n = 1 : N       % for each matrix coefficient
  for i = 1 : m
    for k = 1 : p   % for each element in the matrix
      Z = spalloc(Ng,Ng,1);
      %Z( N*m + i, (n-1)*m + k ) = 1;
      Z( N*p + i, (n-1)*p + k ) = 1;
      As(ia, 2:end) = vec(Z)';
      bs(ia) = -H{N+1-n}(i,k);
      ia = ia+1;
    end
  end
end

% the last diagonal block
for i = 1 : m
  for k = i : m
    Z = spalloc(Ng,Ng,1);
    %Z( N*m + i, N*m + k) = 1;
    Z( N*p + i, N*p + k) = 1;
    As(ia, 2:end) = vec(Z)';
    if i == k
      bs(ia,1) = 1;
    end
    ia = ia + 1;
  end
end

% the main diagonal block trace
for i = 1 : p
  for k = i : p
    Z = spalloc(Ng,Ng,N);
    for z = 0 : N-1
      Z( z*p + i, z*p + k) = 1;
    end
    As(ia, 2:end) = vec(Z)';
    if i == k
      As(ia,1) = - 1;   % for the norm
    end
    ia = ia + 1;
  end
end

% the other diagonal block traces are zero
for n = 1 : N-1
  for i = 1 : p
    for k = 1 : p
      Z = spalloc(Ng,Ng,N-n);
      for z = 0 : N-1-n
        Z( z*p + i, (z+n)*p + k) = 1;
      end
      As(ia, 2:end) = vec(Z)';
      ia = ia + 1;
    end
  end
end

pars.fid = 0;
pars.eps = 1e-8;
[x,y,info] = sedumi(As,bs,cs,Ks,pars);
%info

ninf = sqrt(x(1));
Q = mat(x(2:end));
