function P = pol_mult_vec_c(X, dX, Y, dY, KX, KY, Xvar)

% Multiplication of causal multidimensional polynomials with matrix coefficients,
% represented in vector form.
% Input:
%    X,Y    - vectors representing the polynomials
%    dX,dY  - their degrees (vectors)
%    KX     - size of matrix coefficients for X (default = 1)
%    KY     - size of matrix coefficients for Y (default = KX)
%    Xvar   - if ==0, the variable in a possible CVX call is Y, otherwise is X
%             (this variable is not updated in the function)
% Output:
%    P      - the polynomial X*Y (of degree dX+dY)

% BD 14.01.2009

% check arguments
if nargin < 5
  KX = 1;
end
if nargin < 6
  KY = KX;
end
if nargin < 7
  Xvar = 0;
end
if length(KX)==1
  KX = [KX KX];
end
if length(KY)==1
  KY = [KY KY];
end

K2X = KX(1)*KX(2);   % number of elements in a coef
K2Y = KY(1)*KY(2);
K2P = KX(1)*KY(2);

nvar = length(dX);
if nvar ~= length(dY)
  error('Polynomials have different number of variables')
end

dX = dX(:)';
dY = dY(:)';
dP = dX + dY;   % degree of P

sX = dX + 1;  % size
sY = dY + 1;
sP = dP + 1;

nX = prod(sX);  % number of coefficients
if length(X) ~= nX*K2X
  error('Length of X does not correspond to given dX')
end
nY = prod(sY);
if length(Y) ~= nY*K2Y
  error('Length of Y does not correspond to given dY')
end
nP = prod(sP);

% Build convolution matrix, then multiply
if Xvar       % if X is the variable, build convolution matrix from Y
  C = sparse(nP*K2P, nX*K2X);
  for ix = 1 : nX
    for iy = 1 : nY
      eX = ind2exp_nd(ix, dX);
      eY = ind2exp_nd(iy, dY);
      ip = exp2ind_nd(eX + eY + 1, dP);
      C((ip-1)*K2P+1:ip*K2P,(ix-1)*K2X+1:ix*K2X) = ...
           kron(reshape(Y((iy-1)*K2Y+1:iy*K2Y), KY(1),KY(2))', eye(KX(1)));
    end
  end
  P = C * X;
else          % Y is the variable, build convolution matrix from X
  C = sparse(nP*K2P, nY*K2Y);
  for ix = 1 : nX
    for iy = 1 : nY
      eX = ind2exp_nd(ix, dX);
      eY = ind2exp_nd(iy, dY);
      ip = exp2ind_nd(eX + eY + 1, dP);
      C((ip-1)*K2P+1:ip*K2P,(iy-1)*K2Y+1:iy*K2Y) = ...
           kron(eye(KY(2)), reshape(X((ix-1)*K2X+1:ix*K2X), KX(1), KX(2)));
    end
  end
  P = C * Y;
end


%----------------------------------------------------------
% This code is good, but cannot be used in CVX, as variable Y is updated
% P = zeros(nP*K2,1);   % allocate space for P
% for ix = 1 : nX
%   for iy = 1 : nY
%     B = mat(X((ix-1)*K2+1:ix*K2)) * mat(Y((iy-1)*K2+1:iy*K2));   % product of two coefficients
%     eX = ind2exp_nd(ix, dX);
%     eY = ind2exp_nd(iy, dY);
%     ip = exp2ind_nd(eX + eY + 1, dP);
%     P( (ip-1)*K2+1:ip*K2 ) = P( (ip-1)*K2+1:ip*K2 ) + vec(B);
%   end
% end

