function R = pos_trig_poly( nd, K, D, ctype )

% Generates a vector R representing a trigonometric matrix polynomial
% variable, that is positive on a domain D.
% Input:
%    nd     - the degree of the polynomial, a vector whose length is the
%             number of dimensions. Each element is the degree on that
%             dimension.
%    K      - the size of the matrix coefficients.
%    D      - a description of the domain. If missing, the inequality holds for all omega.
%             Univariate polynomials: D is an interval [a,b]
%               represented by a vector with 2 elements.
%             Multivariate: D is a cell vector, each element being a structure.
%               D{i}.p - symmetric polynomial represented in vector form
%               D{i}.d - is the degree of the polynomial
%    ctype  - a string describing the coefficients
%             'r' or 'real' - real coefficients (default type)
%             'c' or 'complex' - complex coefficients
% Output:
%    R      - a vector representing the positive trigonometric polynomial.
%             Each matrix coefficient is vectorized (on columns).
%             The coefficients belong to a halfplane, which is sequenced on
%             increasing order of dimensions.
%             length(R) = (prod( 2*nd+1 ) + 1) / 2 * K^2 - K*(K-1)/2
%             (the free term is a symmetric matrix).

% BD 13.01.2009

% check arguments !!!
if nargin < 2
  K = 1;
end
if nargin < 3
  D = [];   % global positivity
end
if nargin < 4
  ctype = 'r';   % real coefficients
end
if isempty(D)     % global positivity => polynomial is sos
  R = sos_trig_poly( nd, K );   % call sos function
  return                        % and done
end
nvar = length(nd);  % number of variables
if nvar == 1 && length(D) ~= 2
  error('The variable D must contain two values, representing an interval')
end
real_coef = 1;
if ctype(1) == 'c'
  real_coef = 0;
end

% some numbers
K2 = K*K;           % number of elements in a matrix coefficient
K2s = K*(K+1)/2;    % same, in symmetric part
Rsize = nd + 1;
NS0 = (prod( 2*nd + 1 ) + 1) / 2;  % number of matrix coefficients of S0
LS0 = (NS0-1)*K2 + K2s;            % length of vectorized S0

% for univariate pol., transform interval D into standard description
if nvar == 1
  if real_coef
    c0p = - 0.5 - cos(D(1))*cos(D(2));
    c1p = 0.5*( cos(D(1)) + cos(D(2)) );
    c2p = - 0.25;
    clear D
    D{1}.p = [c0p; c1p; c2p];
    D{1}.d = 2;
  else          % complex coefficients
    c0p = -cos((D(2)-D(1))/2);
    c1p = 0.5*exp(sqrt(-1)*(D(1)+D(2))/2);
    clear D
    D{1}.p = [c0p; c1p];
    D{1}.d = 1;
  end
end

% inspect variable D
nDpol = length(D);    % number of polynomials describing the domain
ND = zeros(1,nDpol);
NS1 = zeros(1,nDpol);
LS1 = zeros(1,nDpol);
for id = 1 : nDpol     % check
  if length(D{id}.d) ~= nvar
    error('Number of variables must be the same for all polynomials')
  end
  ND(id) = (prod( 2*D{id}.d + 1 ) + 1) / 2; % number of distinct coefficients
  if length( D{id}.p ) ~= ND(id)
    error('Number of coefficients of constraint polynomial does no correspond with the degree')
  end
  D{id}.p = D{id}.p(:);
  dS1{id} = nd - D{id}.d;   % degree of sos multiplying this constraint polynomial
  NS1(id) = (prod( 2*dS1{id} + 1 ) + 1) / 2;  % number of matrix coefficients of sos
  LS1(id) = (NS1(id)-1)*K2 + K2s;   % length of vectorized sos
 end
 vnS1 = [0 cumsum(LS1)];  % delimitators in concatenated vectorized sos
 lenS1 = vnS1(end);       % total length of vectorized sos
    
% build convolution matrices for symmetric polynomial multiplications D(z) * S1(z)
for i = 1 : nDpol
  NsS1(i) = prod( 2*dS1{i} + 1 );  % number of matrix coefficients of symmetric sos
  NsD(i) = prod( 2*D{i}.d + 1 );   % number of coefficients of symmetric D
  C1{i} = sparse(NS0, NsS1(i));
  Dsym = [conj(D{i}.p(end:-1:2)); D{i}.p];
  for id = 1 : NsD(i)
    for is = 1 : NsS1(i)
      eD = ind2exp_nd(id, 2*D{i}.d);
      eS = ind2exp_nd(is, 2*dS1{i});
      ip = exp2ind_nd(eD + eS + 1, 2*nd) - NS0 + 1;
      if ip > 0     % compute only "half" of convolution matrix
        C1{i}(ip, is) = Dsym(id);
      end
    end
  end
end

% store block selectors
for i = 1 : K
  for j = 1 : K
    blS{i,j} = zeros(K);
    blS{i,j}(i,j) = 1;
  end
end

% ready for set description based on equality
% R(z) = S0(z) + sum_i Di(z)*Si(z)
cvx_begin set
  if real_coef
    variable R(LS0);      % the polynomial
    variable S0(LS0);     % S0 is alone
    variable S1(lenS1);   % vectorized Si sos
  else
    variable R(LS0) complex;
    variable S0(LS0) complex;
    variable S1(lenS1) complex;
  end
  S0 == sos_trig_poly(nd, K, ctype);
  for id = 1 : nDpol    % declare each sos
    S1(vnS1(id)+1 : vnS1(id+1)) == sos_trig_poly(dS1{id}, K, ctype);
  end

  ki = zeros(nvar,1);
  kb = 1;
  k = 1;
  for kk = 1 : NS0
    % build vectors needed in the linear combination with S1
    for id = 1 : nDpol      % for each polynomial describing the domain
      ca{id} = C1{id}(kb,NS1(id)-1:-1:1); % the anticausal part of current row
      c0{id} = C1{id}(kb,NS1(id));        % central element
      cc{id} = C1{id}(kb,NS1(id)+1:end);  % causal part
    end

    for j = 1 : K     % for the elements of a block, by columns
      for i = 1 : K
        if kb > 1 || i >= j
          % build big vectors multiplying the sos S1
          vcC1 = zeros(1, lenS1);   % will multiply S1
          vaC1 = zeros(1, lenS1);   % will multiply conj(S1)
          for id = 1 : nDpol
            % for multiplication with non-central blocks
            vcC1(vnS1(id) + K2s + 1 : vnS1(id+1)) = kron(cc{id}, vec(blS{i,j}));
            vaC1(vnS1(id) + K2s + 1 : vnS1(id+1)) = kron(ca{id}, vec(blS{j,i}));
            % for the central block
            if i >= j
              vcC1(vnS1(id) + K*(j-1) - j*(j-1)/2 + i) = c0{id};
            else
              vaC1(vnS1(id) + K*(i-1) - i*(i-1)/2 + j) = c0{id};
            end
          end
          %---------------------------------
          R(k) == S0(k) + vcC1 * S1 + vaC1 * conj(S1);
          k = k+1;
        end
      end
    end

    kb = kb + 1;
    ki(1) = ki(1) + 1;        % go to next (poly)index
    for i = 1 : nvar - 1
      if ki(i) == Rsize(i)
        ki(i) = -Rsize(i)+1;
        ki(i+1) = ki(i+1) + 1;
      else
        break;
      end
    end
  end

cvx_end
