function [A, B] = posintlmi_r(n, alfa, beta)

% The matrix coefficients for the LMI corresponding to
% R(w) >= 0, for w in [alfa,beta],
% where R(z) is a symmetric trigonometric polynomial.
% The matrices A, B are so that
% r_k = a_k^T vec(Q) + b_k^T vec(S),
% where Q, S are positive semidefinite matrices
% and a_k^T is the k-th row of A.
% Input:
%   n           - degree of the polynomial
%   alfa, beta  - interval on which the polynomial is nonnegative
%                 (normalized frequencies)
%                 if alfa, beta are not given, then global positivity is assumed
% Return:
%   A, B        - LMI coefficient matrices, as described above
%                 (if alfa, beta are not given, then only A is significant)

% BD 25.01.2006

if nargin==1
  alfa = 0;
  beta = 1;
end

nQ = n+1;     % length of polynomial (and size of Q)
nS = n-1;
nQ2 = nQ * nQ;
nS2 = nS * nS;
a = cos(pi*alfa);
b = cos(pi*beta);
c0 = a*b + 0.5;
c1 = (a+b) / 2;
%c2 = 0.25;

A = sparse(nQ, nQ2);
B = sparse(nQ, nS2);

e = ones(nQ,1);

for k = 0 : n
  A(k+1,:) = vec( spdiags(e,k,nQ,nQ) )';
  B(k+1,:) = -c0 * vec( spdiags(e,k,nS,nS) )' ...
             + c1 * ( vec( spdiags(e,k-1,nS,nS) )' + vec( spdiags(e,k+1,nS,nS) )' ) ...
             - 0.25 * ( vec( spdiags(e,k-2,nS,nS) )' + vec( spdiags(e,k+2,nS,nS) )' );
end
