function R = sos_trig_poly( nd, K, ctype )

% Generates a vector R representing a trigonometric matrix polynomial
% variable, that is sum-of-squares.
% Input:
%    nd     - the degree of the polynomial, a vector whose length is the
%             number of dimensions. Each element is the degree on that
%             dimension.
%    K      - the size of the matrix coefficients
%    ctype  - a string describing the coefficients
%             'r' or 'real' - real coefficients (default type)
%             'c' or 'complex' - complex coefficients
% Output:
%    R      - a vector representing the sum-of-squares trig. polynomial.
%             Each matrix coefficient is vectorized (on columns).
%             The coefficients belong to a halfplane, which is sequenced on
%             increasing order of dimensions.
%             length(R) = (prod( 2*nd+1 ) + 1) / 2 * K^2 - K*(K-1)/2
%             (the free term is a symmetric matrix).

% BD 13.01.2009

% check arguments
if nargin < 2
  K = 1;
end
if nargin < 3
  ctype = 'r';   % real coefficients
end
real_coef = 1;
if ctype(1) == 'c'
  real_coef = 0;
end

% total number of scalar coefficients
N = (prod( 2*nd+1 ) + 1) / 2;     % number of block coefs
M = N * K^2 - K*(K-1)/2;

nvar = length(nd);    % number of dimensions
Rsize = nd + 1;
nG = K * prod(Rsize); % size of Gram matrix

% store block selectors
for i = 1 : K
  for j = 1 : K
    blS{i,j} = zeros(K);
    blS{i,j}(i,j) = 1;
  end
end

% try speeding-up by storing Toeplitz matrices
dmax = max(Rsize);
for i = 1 : 2*dmax-1
  Ts{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
end

cvx_begin set
  if real_coef
    variable R(M);     % the polynomial
    variable Q( nG,nG ) symmetric;
    Q == semidefinite( nG );   % the corresponding Gram matrix
  else
    variable R(M) complex;
    variable Q( nG,nG ) hermitian;
    Q == hermitian_semidefinite( nG );
  end

  % define the coefficients of R by generalized "trace" equalities
  ki = zeros(nvar,1);
  kb = 1;
  k = 1;
  for kk = 1 : N
    T = Ts{ki(1)+dmax}(1:Rsize(1),1:Rsize(1));
    for i = 2 : nvar
      T = kron( Ts{ki(i)+dmax}(1:Rsize(i),1:Rsize(i)), T );
    end
    for j = 1 : K  % for the elements of a block, by columns
      for i = 1 : K
        if kb > 1 || i >= j
          TT = kron(T,blS{i,j});
          R(k) == vec(TT)' * vec(Q);
          k = k+1;
        end
      end
    end

    kb = kb + 1;
    ki(1) = ki(1) + 1;        % go to next (poly)index
    for i = 1 : nvar - 1
      if ki(i) == Rsize(i)
        ki(i) = -Rsize(i)+1;
        ki(i+1) = ki(i+1) + 1;
      else
        break;
      end
    end
  end

cvx_end
