function is_stable = stab_2d_psatz(H, nrel, test1d)

% Stability test for 2D quarter plane polynomial, using
% trigonometric Positivstellensatz.
% Input:
%   H         - coefs of the quarter plane polynomial
%   nrel      - relaxation degree (implicitly equal to degH / 2);
%   test1d    - use also 1D conditions to test stability
% Output:
%   is_stable     - 1 if system is stable, 0 if not

% BD 15.02.2006

n = size(H);    % the order must be even now !!!????
nh = (n-1)/2;   % order of symmetrized polynomial
nh1 = nh+1;
Nh = (prod(n)+1)/2;  % number of distinct coef in symmetrized polynomial
if nargin < 2 || isempty(nrel)
  nrel = nh;
end
if nargin < 3
  test1d = 1;
end
if nrel > nh
  nrel = nh;
end
ns = ceil( (nh + nrel)/2 );   % degree of sum-of-squares factors
ns1 = ns + 1;
Nc = (prod(2*ns+1)+1)/2;      % sizes of Gram matrices
Ns = Nc - 1;
nr = 2 * ns;                  % degree of sos
Nr = (prod(2*nr+1)+1)/2;      % number of distinct coefs in sos
nx = nr - nh;                 % degrees of polynomials multiplying A and B
Nx = (prod(2*nx+1)+1)/2;      % number of distinct coef
%Nh, Nc, Ns, Nr, Nx

% 1D tests
if test1d
  v1 = max(abs(roots(sum(H, 1))));
  v2 = max(abs(roots(sum(H, 2))));
  if max(v1, v2) >= 1 - eps
    is_stable = 0;
    return
  end
end

% build symmetric and antisym parts (A and B)
Hr = H(end:-1:1,end:-1:1);
A = H + Hr;
A(nh1(1),nh1(2)) = H(nh1(1), nh1(2));
B = H - Hr;

v = vec(A);
A = v(Nh:-1:1);
v = vec(B);
B = v(Nh:-1:1);    % cut the zero ???

% numbers of variables and equations
Ks.f = 2*Nx;            % free var: the coefs of X and Y
Ks.s = [Nc Ns];         % Gram pair for sos
nrA = Nr;               % number of equality constraints (one for each coefficient of sos)
ncA = 2*Nx+Nc*Nc+Ns*Ns; % number of (scalar) variables

%As = spalloc( nrA, ncA, floor(nrA*ncA/10) );
As = zeros( nrA, ncA );
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );   % feasibility problem
bs(1) = -1;             % the 1 from the positivstellensatz

% Gram pair for sos
iv = (1:Nc)';
ii = ns(1)+1;
kk = 2*nr(1) + 1 - ns(1);
for i2 = 1 : ns(2)
  iv( ii+1 : ii+nr(1)+1 ) = kk+1 : kk+nr(1)+1;
  ii = ii + nr(1)+1;
  kk = kk + 2*nr(1) + 1;
end
    
% the Q part
sc = 2*Nx;           % start column for Q in As
for ii = 1 : Nc
  for ll = 1 : Nc
    k = iv(ii) + iv(ll) - 1;      % coef affected by sum
    %if k == 1, ad = 0.5; else, ad = 0.25; end
    d = Nc*(ll-1) + ii;
    As(k, sc + d ) = As(k, sc + d ) + 1;
    k = abs(iv(ii) - iv(ll)) + 1; % coef affected by difference
    %if k == 1, ad = 0.5; else, ad = 0.25; end
    As(k, sc + d ) = As(k, sc + d ) + 1;
  end
end

% the S part
sc = 2*Nx+Nc*Nc;     % start column for Q in As
iv = iv(2:end);      % no free term for sines
for ii = 1 : Ns
  for ll = 1 : Ns
    k = iv(ii) + iv(ll) - 1;      % coef affected by sum
    %if k == 1, ad = 0.5; else, ad = 0.25; end
    d = Ns*(ll-1) + ii;
    As(k, sc + d ) = As(k, sc + d ) - 1;
    k = abs(iv(ii) - iv(ll)) + 1; % coef affected by difference
    %if k == 1, ad = 0.5; else, ad = 0.25; end
    As(k, sc + d ) = As(k, sc + d ) + 1;
  end
end

% the X and Y terms
ivx = (1:Nx)';       % vector of indices for coefs of X (linearization)
ii = nx(1)+1;
ni = 2*nx(1)+1;      % number of coefs on a "row"
kk = 2*nr(1) + 1 - nx(1);
for i2 = 1 : nx(2)
  ivx( ii+1 : ii+ni ) = kk+1 : kk+ni;
  ii = ii + ni;
  kk = kk + 2*nr(1) + 1;
end
    
iva = (1:Nh)';       % same for A
ii = nh(1)+1;
ni = 2*nh(1)+1;      % number of coefs on a "row"
kk = 2*nr(1) + 1 - nh(1);
for i2 = 1 : nh(2)
  iva( ii+1 : ii+ni ) = kk+1 : kk+ni;
  ii = ii + ni;
  kk = kk + 2*nr(1) + 1;
end

for ii = 1 : Nx
  for ll = 1 : Nh
    k = ivx(ii) + iva(ll) - 1;      % coef affected by sum
    As(k, ii) = As(k, ii) + A(ll);  % in AX
    As(k, Nx+ii) = As(k, Nx+ii) - B(ll); % in BY
    
    k = abs(ivx(ii) - iva(ll)) + 1;   % coef affected by difference
    As(k, ii) = As(k, ii) + A(ll);  % in AX
    As(k, Nx+ii) = As(k, Nx+ii) + B(ll); % in BY
  end
end

% call SeDuMi
%pars.fid = 0;     % prepare SeDuMi parameters and call
pars.eps = 1e-10;
[x,y,info] = sedumi(As,bs,cs,Ks,pars); % call SeDuMi
info              % display run statistics

is_stable = 1 - info.pinf;
