function [is_stable, mineig, gap, minpol] = stab_nd_syst(polcoef, use_divisors)

% DeCarlo-Strintzis stability test for discrete nD-polynomial A(z):
% check if A(z) not equal to zero on the unit n-circle.
% This is done by maximizing the minimum eigenvalue of Gram
% matrices associated with G(z) = A(z)A(z^{-1}).
% Input:
%   polcoef       - n-dimensional matrix of coefficients
%                   (the polynomial A(z) is expressed with positive powers of z_k;
%                   the free term is polcoef(1,...,1))
%   use_divisors  - heuristic used for complexity reduction
%                   0 - complete problem (no reduction)
%                   1 - basic (use only monomials with nonzero coefficients)
%                   2 - "divisors" (use only divisors of the 
%                       monomials with nonzero coefficients)
% Output:
%   is_stable     - 1 if system is stable, 0 if not
%   mineig        - minimum eigenvalue of most positive Gram matrix associated to G(z)
%   gap           - gap (between primal and dual) achieved by the SDP solver
%   minpol        - minimum value of |A(z)| on the unit circle
%
% The numerical stability test is:
%   if mineig > 100 * gap
%       the system is stable
%   else
%       the system is not stable (or stable, but very near instability)

% BD 26.08.2003

ndeg = size( polcoef );    % max degrees (+1) of variables in polynomial
nvar = length( ndeg );     % number of variables

% check one-dimensional stability conditions
for i = 1 : nvar
  v = sumdim(polcoef, i);
  if max(abs(roots(v))) >= 0.9999999
    is_stable = 0;
    mineig = 0;
    gap = 0;
    minpol = 0;
    return
  end
end

if nargin < 2
  use_divisors = 0;
end

% identify monomials that are used in SDP optimization
if use_divisors
  pol_used = zeros(prod(ndeg),1);
  inz = find( polcoef ~= 0 );
  nz = length(inz);
  refd = [1 cumprod(ndeg(1:end-1))];
  pol_used(inz) = 1;              % mark coefficients of actual monomials
  if use_divisors == 2            % find divisors of actual monomials
    for i = 1 : nz
      ip = get_deg(inz(i), ndeg); % vector of monomial powers
      ki = zeros(1,nvar);
      nip = prod(ip);             % number of divisors
      for k = 1 : nip
        pol_used(ki*refd' + 1) = 1;  % mark useful coefficient !
        ki(1) = ki(1) + 1;         % go to next (poly)index
        for ii = 1 : nvar - 1
          if ki(ii) == ip(ii)
            ki(ii) = 0;
            ki(ii+1) = ki(ii+1) + 1;
          else
            break;
          end
        end
        k = k+1;
      end
    end
  end
  
  coef_used = find(pol_used==1)';
  nr_used_zeros = length(coef_used)
end

% start multidimensional test
vcoef = vec(polcoef);       % vector of coefficients
if use_divisors
  vcoef = vcoef( coef_used );
end
Gcoef = vec(vcoef * vcoef'); % symmetric product polynomial
nv = length(vcoef);
nd2 = 2*ndeg - 1;
M = prod( nd2 );
N = (M+1)/2;                % number of coefs in symmetric product polynomial

% Sedumi parameters
nrA = N;                    % number of equality constraints
ncA = 1 + nv*nv;            % number of variables

Ks.l = 1;                   % the eigenvalue 
% tried Ks.f=1 (taken real, although it's positive) but it's not necessary
Ks.s = nv;

As = spalloc( nrA, ncA, 2*nv*nv );  % very approximate allocation
bs = zeros( nrA, 1 );
cs = zeros( ncA, 1 );
cs(1) = -1;                 % maximize the eigenvalue

%======================================================
if use_divisors
  vnz_tr = zeros(N,1);        % vector of nonzero traces in reduced Gram matrix
  vnz_tr(1) = 1;              % main diagonal always present
  for i1 = 1 : nr_used_zeros
    for i2 = i1+1 : nr_used_zeros
      ip1 = get_deg(coef_used(i1), ndeg);
      ip2 = get_deg(coef_used(i2), ndeg);
      ip = ip2 - ip1;
      kk = exp2ind_halfsp(ip2 - ip1, ndeg);
      vnz_tr(kk) = 1;
    end
  end
else
  vnz_tr = ones(N,1);
end

%======================================================
%-------------------------------------
% try speeding-up by storing Toeplitz matrices
dmax = max(ndeg);
for i = 1 : 2*dmax-1
  Ts{i} = spdiags( ones(dmax,1), i-dmax, dmax, dmax );
end
%-------------------------------------
% generalized "trace" equalities
ki = zeros(nvar,1);
k = 1;
for kk = 1 : N
  if vnz_tr(kk);
    T = Ts{ki(1)+dmax}(1:ndeg(1),1:ndeg(1));
    for i = 2 : nvar
      %T = kron( spdiags(ones(ndeg(i),1),-ki(i), ndeg(i), ndeg(i)), T);
      T = kron( Ts{ki(i)+dmax}(1:ndeg(i),1:ndeg(i)), T );
    end
    if use_divisors
      T = T(coef_used, coef_used);  % reduce size
    end
    if (use_divisors==0) | length(find(T))
      T = (T+T')/2;
      As(k,1) = trace(T);       % eigenvalue coefficient
      T = vec(T);
      As(k, 2:end) = T';        % "trace" matrix
      bs(k) = T'* Gcoef;        % value of product polynomial coefficient
      k = k+1;
    end
  end
  ki(1) = ki(1) + 1;        % go to next (poly)index
  for i = 1 : nvar - 1
    if ki(i) == ndeg(i)
      ki(i) = -ndeg(i)+1;
      ki(i+1) = ki(i+1) + 1;
    else
      break;
    end
  end
end
%spy(As)

if use_divisors             % update dimensions
  nrA = k-1;
  As = As(1:nrA, :);
  bs = bs(1:nrA);
end
%nrA, ncA
%======================================================

% call Sedumi
pars.fid = 0;
pars.eps = 1e-10;

[x,y,info] = sedumi(As,bs,cs,Ks,pars);

mineig = x(1);
Q = mat(x(2:end));
info
gap = abs(cs'*x - bs'*y);
minpol = sqrt( mineig * size(Q,1) );
if mineig > 100*gap
  is_stable = 1;
else
  is_stable = 0;
end

%---------------------------------
function coef = sumdim(coef,dim)
% sum all coefficients of nD matrix coef projecting on dimension dim

ns = size(coef);
nd = length(ns);      % number of dimensions
for i = 1 : nd
  if i ~= dim
    coef = sum(coef, i);
  end
end
coef = coef(:)';
